# -*- perl -*-

use strict;

package CHIRP::Foundry::BigIron;

use lib "/usr/local/cricket/lib";
use lib::listInterfaces;
use snmpUtils;


sub rehash (@) {
  my @a = @_;
  my $b = {};
  my $row;
    
  foreach $row (@a) {
    my ($index, $content) = split (/:/, $row, 2);
    my ($i, $j) = split (/\./, $index, 2);
    $b->{$j}->{$i} = $content;
  }
  
  return $b;    
}

my $sysName = ".1.3.6.1.2.1.1.5.0";

sub GenConfig {
  my($hostname, $snmp_community, $snmp_port, $options) = @_;

  $snmp_community = 'public' unless defined($snmp_community);
  $snmp_port = '161' unless defined($snmp_port);

  my($snmp) = "$snmp_community\@$hostname:$snmp_port";
  my($config) = {};

  my ($sysname) = snmpUtils::snmpget ($snmp, $sysName);
  
  my $row;
  
  $config->{target}->{"--default--"}->{"rrd-datafile"} =	'%dataDir%/%auto-target-name%.rrd';
  # $config->{target}->{"--default--"}->{"router"} =	 	'%auto-target-name%';
  $config->{target}->{"--default--"}->{"router"} =	 	"\"$hostname\"";
  $config->{target}->{"--default--"}->{"snmp-host"} =	 	'%router%';
  $config->{target}->{"--default--"}->{"snmp-community"} = "$snmp_community";
  $config->{target}->{"--default--"}->{"snmp-port"} = "$snmp_port";
  $config->{target}->{"--default--"}->{"target-type"} =	 	'undef';
  $config->{target}->{"--default--"}->{"snmp-version"} = 	'2c';
  $config->{target}->{"--default--"}->{"directory-desc"} =	"\"[Foundry BigIron] Statistics for $hostname\ ($sysname)\"";

  $config->{oid}->{"snAgGblDynMemFree"} 	=	'.1.3.6.1.4.1.1991.1.1.2.1.55.0';
  $config->{oid}->{"snAgGblDynMemTotal"} 	=	'.1.3.6.1.4.1.1991.1.1.2.1.54.0';
  $config->{oid}->{"snAgGblDynMemUtil"} 	=	'.1.3.6.1.4.1.1991.1.1.2.1.53.0';

  $config->{oid}->{"snAgGblCpuUtil1MinAvg"}	= '.1.3.6.1.4.1.1991.1.1.2.1.52.0';

  $config->{datasource}->{"--default--"}->{"rrd-ds-type"} 	= "GAUGE";
  $config->{datasource}->{"--default--"}->{"rrd-heartbeat"} 	= "1800";
  $config->{datasource}->{"--default--"}->{"rrd-min"}		= "undef";
  $config->{datasource}->{"--default--"}->{"rrd-max"}		= "undef";

  $config->{datasource}->{"snAgGblCpuUtil1MinAvg"}->{"ds-source"} = 'snmp://%snmp%/snAgGblCpuUtil1MinAvg';
  $config->{datasource}->{"snAgGblDynMemFree"}->{"ds-source"} = 'snmp://%snmp%/snAgGblDynMemFree';
  $config->{datasource}->{"snAgGblDynMemTotal"}->{"ds-source"} = 'snmp://%snmp%/snAgGblDynMemTotal';
  $config->{datasource}->{"snAgGblDynMemUtil"}->{"ds-source"} = 'snmp://%snmp%/snAgGblDynMemUtil';

  $config->{targettype}->{"CPU"}->{"ds"} = '"snAgGblCpuUtil1MinAvg"';
  $config->{targettype}->{"CPU"}->{"view"} = '"CPU Utilization: snAgGblCpuUtil1MinAvg"';

  $config->{targettype}->{"FoundryMemory"}->{"ds"} = '"snAgGblDynMemFree, snAgGblDynMemTotal, snAgGblDynMemUtil"';
  $config->{targettype}->{"FoundryMemory"}->{"view"} = '"Absolute: snAgGblDynMemFree snAgGblDynMemTotal, Relative: snAgGblDynMemUtil"';

  $config->{graph}->{"snAgGblDynMemUtil"}->{"color"} = 'dark-green';
  $config->{graph}->{"snAgGblDynMemUtil"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"snAgGblDynMemUtil"}->{"y-axis"} = '"Percent"';
  $config->{graph}->{"snAgGblDynMemUtil"}->{"legend"} = '"Dynamic Memory Utilization"';
  $config->{graph}->{"snAgGblDynMemUtil"}->{"units"} = '"%"';
  $config->{graph}->{"snAgGblDynMemUtil"}->{"si-units"} = 'false';
  $config->{graph}->{"snAgGblDynMemUtil"}->{"y-min"} = '0';

  $config->{graph}->{"snAgGblCpuUtil1MinAvg"}->{"color"} = 'dark-green';
  $config->{graph}->{"snAgGblCpuUtil1MinAvg"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"snAgGblCpuUtil1MinAvg"}->{"y-axis"} = '"Percent"';
  $config->{graph}->{"snAgGblCpuUtil1MinAvg"}->{"legend"} = '"Relative Memory Utilization"';
  $config->{graph}->{"snAgGblCpuUtil1MinAvg"}->{"units"} = '"%"';
  $config->{graph}->{"snAgGblCpuUtil1MinAvg"}->{"si-units"} = 'false';
  $config->{graph}->{"snAgGblCpuUtil1MinAvg"}->{"y-min"} = '0';

  $config->{graph}->{"snAgGblDynMemTotal"}->{"color"} = 'dark-green';
  $config->{graph}->{"snAgGblDynMemTotal"}->{"draw-as"} = 'AREA';
  $config->{graph}->{"snAgGblDynMemTotal"}->{"y-axis"} = '"bytes"';
  $config->{graph}->{"snAgGblDynMemTotal"}->{"legend"} = '"Total Memory"';
  $config->{graph}->{"snAgGblDynMemTotal"}->{"units"} = '"bytes"';
  $config->{graph}->{"snAgGblDynMemTotal"}->{"si-units"} = 'false';
  $config->{graph}->{"snAgGblDynMemTotal"}->{"y-min"} = '0';

  $config->{graph}->{"snAgGblDynMemFree"}->{"color"} = 'blue';
  $config->{graph}->{"snAgGblDynMemFree"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"snAgGblDynMemFree"}->{"y-axis"} = '"Bytes"';
  $config->{graph}->{"snAgGblDynMemFree"}->{"legend"} = '"Free Memory"';
  $config->{graph}->{"snAgGblDynMemFree"}->{"units"} = '"bytes"';
  $config->{graph}->{"snAgGblDynMemFree"}->{"si-units"} = 'false';
  $config->{graph}->{"snAgGblDynMemFree"}->{"y-min"} = '0';

  CHIRP::listInterfaces::listInterfaces ($config, $hostname, $snmp_community, 1, 1, 0, 0, "bits", []);

  $config->{target}->{"$hostname.CPU"}->{"long-desc"} = '"[CPU] Device CPU Utilization"';
  $config->{target}->{"$hostname.CPU"}->{"short-desc"} = '"CPU Utilization"';
  $config->{target}->{"$hostname.CPU"}->{"target-type"} = '"CPU"';

  $config->{target}->{"$hostname.Memory"}->{"target-type"} = '"FoundryMemory"';
  $config->{target}->{"$hostname.Memory"}->{"short-desc"} = '"Memory"';
  $config->{target}->{"$hostname.Memory"}->{"long-desc"} = '"[MEM] Memory Utilization"';
  
  return $config;
 
}

1; 
