# -*- perl -*-

use strict;

package CHIRP::Foundry::ServerIron;

use lib "/usr/local/cricket/lib";
use lib::listInterfaces;
# use lib::foundyTemp;
use snmpUtils;

my %IOS_ID = (
  '3.1' => 'snSI', # Stackable ServerIron
  '3.2' => 'snSIXL', # Stackable ServerIronXL
  '3.3' => 'snSIXLTCS', # Stackable ServerIronXL TCS
  '18.1' => 'snSI400', # ServerIron 400
  '19.1' => 'snSI800', # ServerIron800
  '20.1' => 'snSI1500' # ServerIron 1500
);

my $sysObjectOID = '1.3.6.1.2.1.1.2.0';
my $startchassisOID = '1.3.6.1.4.1.1991.1.3.';

my $snL4VirtualServerTableOID =		".1.3.6.1.4.1.1991.1.1.4.2.1.1";
my $snL4RealServerTableOID =		".1.3.6.1.4.1.1991.1.1.4.3.1.1";
					 
my $snL4VirtualServerPortTableOID =	".1.3.6.1.4.1.1991.1.1.4.4.1.1";
my $snL4RealServerPortTableOID =	".1.3.6.1.4.1.1991.1.1.4.5.1.1";

my $sysName = ".1.3.6.1.2.1.1.5.0";

sub GenConfig {
  my($hostname, $snmp_community, $snmp_port, $options) = @_;

  $snmp_community = 'public' unless defined($snmp_community);
  $snmp_port = '161' unless defined($snmp_port);

  my($snmp) = "$snmp_community\@$hostname:$snmp_port";

  my ($chassis) = snmpUtils::get ($snmp, $sysObjectOID);
  #print "Chassis = $chassis\n";
  my ($chassis_reg) = ($chassis =~ /$startchassisOID(\d+.\d+)/);
  #print "Chassis Reg = $chassis_reg\n";
  my $chassisDesc = (defined $IOS_ID{$chassis_reg})?$IOS_ID{$chassis_reg}:"Unknown Chassis Type";
  #print "Chassis Desc = $chassisDesc\n";

  my $sup64bit = ($chassisDesc !~ /snSIXL/);

  my($config) = {};
  
  my $row;

  my ($sysname) = snmpUtils::snmpget ($snmp, $sysName);

  my $result = rehash (snmpUtils::snmpwalk ($snmp, $snL4RealServerTableOID));
  my %virtuals;
  my %reals;
  
  $config->{target}->{"--default--"}->{"rrd-datafile"} =	'%dataDir%/%auto-target-name%.rrd';
  # $config->{target}->{"--default--"}->{"router"} =	 	'%auto-target-name%';
  $config->{target}->{"--default--"}->{"router"} =	 	"\"$hostname\"";
  $config->{target}->{"--default--"}->{"snmp-host"} =	 	'%router%';
  $config->{target}->{"--default--"}->{"target-type"} =	 	'undef';
  $config->{target}->{"--default--"}->{"snmp-version"} =        '2c';
  $config->{target}->{"--default--"}->{"snmp-community"} = "$snmp_community";
  $config->{target}->{"--default--"}->{"snmp-port"} = "$snmp_port";
  $config->{target}->{"--default--"}->{"directory-desc"} =	
                            "\"[Foundry ServerIron] Layer4+ Statistics for $hostname ($sysname)\"";

  $config->{oid}->{"snL4RealServerStatisticReceivePkts"} =	'.1.3.6.1.4.1.1991.1.1.4.23.1.1.3';
  $config->{oid}->{"snL4RealServerStatisticTransmitPkts"} =	'.1.3.6.1.4.1.1991.1.1.4.23.1.1.4';
  $config->{oid}->{"snL4RealServerStatisticCurConnections"} =	'.1.3.6.1.4.1.1991.1.1.4.23.1.1.5';
  $config->{oid}->{"snL4RealServerStatisticTotalConnections"} =	'.1.3.6.1.4.1.1991.1.1.4.23.1.1.6';
  $config->{oid}->{"snL4RealServerStatisticFailTime"} =		'.1.3.6.1.4.1.1991.1.1.4.23.1.1.12';
  $config->{oid}->{"snL4RealServerStatisticPeakConnections"} =	'.1.3.6.1.4.1.1991.1.1.4.23.1.1.13';

  $config->{oid}->{"snL4RealServerPortStatisticState"} 		= '.1.3.6.1.4.1.1991.1.1.4.24.1.1.5';
  $config->{oid}->{"snL4RealServerPortStatusFailTime"} 		= '.1.3.6.1.4.1.1991.1.1.4.24.1.1.6';
  $config->{oid}->{"snL4RealServerPortStatusCurrentConnection"} = '.1.3.6.1.4.1.1991.1.1.4.24.1.1.7';
  $config->{oid}->{"snL4RealServerPortStatusTotalConnection"}	= '.1.3.6.1.4.1.1991.1.1.4.24.1.1.8';
  $config->{oid}->{"snL4RealServerPortStatusPeakConnection"}	= '.1.3.6.1.4.1.1991.1.1.4.24.1.1.13';
  $config->{oid}->{"snL4RealServerPortStatusRxPkts"}		= '.1.3.6.1.4.1.1991.1.1.4.24.1.1.9';
  $config->{oid}->{"snL4RealServerPortStatusTxPkts"}		= '.1.3.6.1.4.1.1991.1.1.4.24.1.1.10';
  $config->{oid}->{"snL4RealServerPortStatusRxBytes"}		= '.1.3.6.1.4.1.1991.1.1.4.24.1.1.11';
  $config->{oid}->{"snL4RealServerPortStatusTxBytes"}		= '.1.3.6.1.4.1.1991.1.1.4.24.1.1.12';

  $config->{oid}->{"snL4VirtualServerStatisticReceivePkts"}	= '.1.3.6.1.4.1.1991.1.1.4.25.1.1.3';
  $config->{oid}->{"snL4VirtualServerStatisticTransmitPkts"}	= '.1.3.6.1.4.1.1991.1.1.4.25.1.1.4';
  $config->{oid}->{"snL4VirtualServerStatisticTotalConnections"}	= '.1.3.6.1.4.1.1991.1.1.4.25.1.1.5';

  $config->{oid}->{"snL4VirtualServerPortStatisticCurrentConnection"} = '.1.3.6.1.4.1.1991.1.1.4.26.1.1.4';
  $config->{oid}->{"snL4VirtualServerPortStatisticTotalConnection"}	= '.1.3.6.1.4.1.1991.1.1.4.26.1.1.5';
  $config->{oid}->{"snL4VirtualServerPortStatisticPeakConnection"}	= '.1.3.6.1.4.1.1991.1.1.4.26.1.1.6';

  $config->{oid}->{"snAgGblCpuUtil1MinAvg"}			= '.1.3.6.1.4.1.1991.1.1.2.1.52.0';

  $config->{datasource}->{"--default--"}->{"rrd-ds-type"} 	= "GAUGE";
  $config->{datasource}->{"--default--"}->{"rrd-heartbeat"} 	= "1800";
  $config->{datasource}->{"--default--"}->{"rrd-min"}		= "undef";
  $config->{datasource}->{"--default--"}->{"rrd-max"}		= "undef";

  $config->{datasource}->{"snAgGblCpuUtil1MinAvg"}->{"ds-source"} = 'snmp://%snmp%/snAgGblCpuUtil1MinAvg';

  $config->{datasource}->{"snL4RealServerStatisticReceivePkts"}->{"ds-source"} = 'snmp://%snmp%/snL4RealServerStatisticReceivePkts.%auto-target-name%';
  $config->{datasource}->{"snL4RealServerStatisticReceivePkts"}->{"rrd-ds-type"} = 'COUNTER';

  $config->{datasource}->{"snL4RealServerStatisticTransmitPkts"}->{"ds-source"} = 'snmp://%snmp%/snL4RealServerStatisticTransmitPkts.%auto-target-name%';
  $config->{datasource}->{"snL4RealServerStatisticTransmitPkts"}->{"rrd-ds-type"} = 'COUNTER';

  $config->{datasource}->{"snL4RealServerStatisticCurConnections"}->{"ds-source"} = 'snmp://%snmp%/snL4RealServerStatisticCurConnections.%auto-target-name%';
  $config->{datasource}->{"snL4RealServerStatisticTotalConnections"}->{"ds-source"} = 'snmp://%snmp%/snL4RealServerStatisticTotalConnections.%auto-target-name%';
  $config->{datasource}->{"snL4RealServerStatisticTotalConnections"}->{"rrd-ds-type"} = 'COUNTER';

  $config->{datasource}->{"snL4RealServerStatisticFailTime"}->{"ds-source"} = 'snmp://%snmp%/snL4RealServerStatisticFailTime.%auto-target-name%';
  $config->{datasource}->{"snL4RealServerStatisticPeakConnections"}->{"ds-source"} = 'snmp://%snmp%/snL4RealServerStatisticPeakConnections.%auto-target-name%';
  $config->{datasource}->{"snL4RealServerPortStatisticState"}->{"ds-source"} = 'snmp://%snmp%/snL4RealServerPortStatisticState.%auto-target-name%';
  $config->{datasource}->{"snL4RealServerPortStatusFailTime"}->{"ds-source"} = 'snmp://%snmp%/snL4RealServerPortStatusFailTime.%auto-target-name%';
  $config->{datasource}->{"snL4RealServerPortStatusCurrentConnection"}->{"ds-source"} = 'snmp://%snmp%/snL4RealServerPortStatusCurrentConnection.%auto-target-name%';
  $config->{datasource}->{"snL4RealServerPortStatusTotalConnection"}->{"ds-source"} = 'snmp://%snmp%/snL4RealServerPortStatusTotalConnection.%auto-target-name%';
  $config->{datasource}->{"snL4RealServerPortStatusTotalConnection"}->{"rrd-ds-type"} = 'COUNTER';
  $config->{datasource}->{"snL4RealServerPortStatusPeakConnection"}->{"ds-source"} = 'snmp://%snmp%/snL4RealServerPortStatusPeakConnection.%auto-target-name%';
  $config->{datasource}->{"snL4RealServerPortStatusRxPkts"}->{"ds-source"} = 'snmp://%snmp%/snL4RealServerPortStatusRxPkts.%auto-target-name%';
  $config->{datasource}->{"snL4RealServerPortStatusRxPkts"}->{"rrd-ds-type"} = 'COUNTER';
  $config->{datasource}->{"snL4RealServerPortStatusTxPkts"}->{"ds-source"} = 'snmp://%snmp%/snL4RealServerPortStatusTxPkts.%auto-target-name%';
  $config->{datasource}->{"snL4RealServerPortStatusTxPkts"}->{"rrd-ds-type"} = 'COUNTER';
  $config->{datasource}->{"snL4RealServerPortStatusRxBytes"}->{"ds-source"} = 'snmp://%snmp%/snL4RealServerPortStatusRxBytes.%auto-target-name%';
  $config->{datasource}->{"snL4RealServerPortStatusRxBytes"}->{"rrd-ds-type"} = 'COUNTER';
  $config->{datasource}->{"snL4RealServerPortStatusTxBytes"}->{"ds-source"} = 'snmp://%snmp%/snL4RealServerPortStatusTxBytes.%auto-target-name%';
  $config->{datasource}->{"snL4RealServerPortStatusTxBytes"}->{"rrd-ds-type"} = 'COUNTER';
  $config->{datasource}->{"snL4VirtualServerStatisticReceivePkts"}->{"ds-source"} = 'snmp://%snmp%/snL4VirtualServerStatisticReceivePkts.%auto-target-name%';
  $config->{datasource}->{"snL4VirtualServerStatisticReceivePkts"}->{"rrd-ds-type"} = 'COUNTER';
  $config->{datasource}->{"snL4VirtualServerStatisticTransmitPkts"}->{"ds-source"} = 'snmp://%snmp%/snL4VirtualServerStatisticTransmitPkts.%auto-target-name%';
  $config->{datasource}->{"snL4VirtualServerStatisticTransmitPkts"}->{"rrd-ds-type"} = 'COUNTER';
  $config->{datasource}->{"snL4VirtualServerStatisticTotalConnections"}->{"ds-source"} = 'snmp://%snmp%/snL4VirtualServerStatisticTotalConnections.%auto-target-name%';
  $config->{datasource}->{"snL4VirtualServerStatisticTotalConnections"}->{"rrd-ds-type"} = 'COUNTER';
  $config->{datasource}->{"snL4VirtualServerPortStatisticCurrentConnection"}->{"ds-source"} = 'snmp://%snmp%/snL4VirtualServerPortStatisticCurrentConnection.%auto-target-name%';
  $config->{datasource}->{"snL4VirtualServerPortStatisticTotalConnection"}->{"ds-source"} = 'snmp://%snmp%/snL4VirtualServerPortStatisticTotalConnection.%auto-target-name%';
  $config->{datasource}->{"snL4VirtualServerPortStatisticTotalConnection"}->{"rrd-ds-type"} = 'COUNTER';
  $config->{datasource}->{"snL4VirtualServerPortStatisticPeakConnection"}->{"ds-source"} = 'snmp://%snmp%/snL4VirtualServerPortStatisticPeakConnection.%auto-target-name%';

  $config->{targettype}->{"CPU"}->{"ds"} = '"snAgGblCpuUtil1MinAvg"';
  $config->{targettype}->{"CPU"}->{"view"} = '"CPU Utilization: snAgGblCpuUtil1MinAvg"';

  $config->{targettype}->{"real_server"}->{"ds"} = '"snL4RealServerStatisticReceivePkts, snL4RealServerStatisticTransmitPkts, snL4RealServerStatisticCurConnections, snL4RealServerStatisticTotalConnections, snL4RealServerStatisticFailTime, snL4RealServerStatisticPeakConnections"';
  $config->{targettype}->{"real_server"}->{"view"} = '"Packets: snL4RealServerStatisticReceivePkts snL4RealServerStatisticTransmitPkts,Connections: snL4RealServerStatisticCurConnections snL4RealServerStatisticTotalConnections snL4RealServerStatisticPeakConnections,Fail_Time: snL4RealServerStatisticFailTime,"';

  $config->{targettype}->{"real_server_port"}->{"ds"} = '"snL4RealServerPortStatusFailTime, snL4RealServerPortStatusCurrentConnection, snL4RealServerPortStatusTotalConnection, snL4RealServerPortStatusPeakConnection, snL4RealServerPortStatusRxPkts, snL4RealServerPortStatusTxPkts, snL4RealServerPortStatusRxBytes, snL4RealServerPortStatusTxBytes"';
  $config->{targettype}->{"real_server_port"}->{"view"} = '"Packets: snL4RealServerPortStatusRxPkts snL4RealServerPortStatusTxPkts,Connections: snL4RealServerPortStatusCurrentConnection snL4RealServerPortStatusTotalConnection snL4RealServerPortStatusPeakConnection,Bytes: snL4RealServerPortStatusRxBytes snL4RealServerPortStatusTxBytes,Fail_Time: snL4RealServerPortStatusFailTime"';

  $config->{targettype}->{"virtual_server"}->{"ds"} = '"snL4VirtualServerStatisticReceivePkts, snL4VirtualServerStatisticTransmitPkts, snL4VirtualServerStatisticTotalConnections"';
  $config->{targettype}->{"virtual_server"}->{"view"} = '"Packets: snL4VirtualServerStatisticReceivePkts snL4VirtualServerStatisticTransmitPkts,Connections: snL4VirtualServerStatisticTotalConnections,"';

  $config->{targettype}->{"virtual_server_port"}->{"ds"} = '"snL4VirtualServerPortStatisticCurrentConnection, snL4VirtualServerPortStatisticTotalConnection, snL4VirtualServerPortStatisticPeakConnection"';
  $config->{targettype}->{"virtual_server_port"}->{"view"} = '"Connections: snL4VirtualServerPortStatisticTotalConnection snL4VirtualServerPortStatisticCurrentConnection snL4VirtualServerPortStatisticPeakConnection,"';

  $config->{graph}->{"snAgGblCpuUtil1MinAvg"}->{"color"} = 'dark-green';
  $config->{graph}->{"snAgGblCpuUtil1MinAvg"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"snAgGblCpuUtil1MinAvg"}->{"y-axis"} = '"Percent"';
  $config->{graph}->{"snAgGblCpuUtil1MinAvg"}->{"legend"} = '"CPU Utilization Average"';
  $config->{graph}->{"snAgGblCpuUtil1MinAvg"}->{"units"} = '"%"';
  $config->{graph}->{"snAgGblCpuUtil1MinAvg"}->{"si-units"} = 'false';
  $config->{graph}->{"snAgGblCpuUtil1MinAvg"}->{"y-min"} = '0';
  $config->{graph}->{"snAgGblCpuUtil1MinAvg"}->{"y-max"} = '100';

  $config->{graph}->{"snL4RealServerStatisticReceivePkts"}->{"color"} = 'dark-green';
  $config->{graph}->{"snL4RealServerStatisticReceivePkts"}->{"draw-as"} = 'AREA';
  $config->{graph}->{"snL4RealServerStatisticReceivePkts"}->{"y-axis"} = '"Packets"';
  $config->{graph}->{"snL4RealServerStatisticReceivePkts"}->{"legend"} = '"Packets received by the RealServer"';
  $config->{graph}->{"snL4RealServerStatisticReceivePkts"}->{"units"} = '"Pkts"';
  $config->{graph}->{"snL4RealServerStatisticReceivePkts"}->{"si-units"} = 'false';
  $config->{graph}->{"snL4RealServerStatisticReceivePkts"}->{"y-min"} = '0';

  $config->{graph}->{"snL4RealServerStatisticTransmitPkts"}->{"color"} = 'blue';
  $config->{graph}->{"snL4RealServerStatisticTransmitPkts"}->{"y-axis"} = '"Packets"';
  $config->{graph}->{"snL4RealServerStatisticTransmitPkts"}->{"legend"} = '"Packets transmit by the RealServer"';
  $config->{graph}->{"snL4RealServerStatisticTransmitPkts"}->{"units"} = '"Pkts"';
  $config->{graph}->{"snL4RealServerStatisticTransmitPkts"}->{"si-units"} = 'false';
  $config->{graph}->{"snL4RealServerStatisticTransmitPkts"}->{"y-min"} = '0';

  $config->{graph}->{"snL4RealServerStatisticTotalConnections"}->{"color"} = 'dark-green';
  $config->{graph}->{"snL4RealServerStatisticTotalConnections"}->{"draw-as"} = 'AREA';
  $config->{graph}->{"snL4RealServerStatisticTotalConnections"}->{"y-axis"} = '"Connection"';
  $config->{graph}->{"snL4RealServerStatisticTotalConnections"}->{"legend"} = '"Total connection on the RealServer"';
  $config->{graph}->{"snL4RealServerStatisticTotalConnections"}->{"units"} = '"Co"';
  $config->{graph}->{"snL4RealServerStatisticTotalConnections"}->{"si-units"} = 'false';
  $config->{graph}->{"snL4RealServerStatisticTotalConnections"}->{"y-min"} = '0';

  $config->{graph}->{"snL4RealServerStatisticCurConnections"}->{"color"} = 'blue';
  $config->{graph}->{"snL4RealServerStatisticCurConnections"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"snL4RealServerStatisticCurConnections"}->{"y-axis"} = '"Connection"';
  $config->{graph}->{"snL4RealServerStatisticCurConnections"}->{"legend"} = '"Current connection on the RealServer"';
  $config->{graph}->{"snL4RealServerStatisticCurConnections"}->{"units"} = '"Co"';
  $config->{graph}->{"snL4RealServerStatisticCurConnections"}->{"si-units"} = 'false';
  $config->{graph}->{"snL4RealServerStatisticCurConnections"}->{"y-min"} = '0';

  $config->{graph}->{"snL4RealServerStatisticPeakConnections"}->{"color"} = 'red';
  $config->{graph}->{"snL4RealServerStatisticPeakConnections"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"snL4RealServerStatisticPeakConnections"}->{"y-axis"} = '"Connection"';
  $config->{graph}->{"snL4RealServerStatisticPeakConnections"}->{"legend"} = '"Peak connection on the RealServer"';
  $config->{graph}->{"snL4RealServerStatisticPeakConnections"}->{"units"} = '"Co"';
  $config->{graph}->{"snL4RealServerStatisticPeakConnections"}->{"si-units"} = 'false';
  $config->{graph}->{"snL4RealServerStatisticPeakConnections"}->{"y-min"} = '0';

  $config->{graph}->{"snL4RealServerStatisticFailTime"}->{"color"} = 'orange';
  $config->{graph}->{"snL4RealServerStatisticFailTime"}->{"draw-as"} = 'AREA';
  $config->{graph}->{"snL4RealServerStatisticFailTime"}->{"y-axis"} = '"Seconds"';
  $config->{graph}->{"snL4RealServerStatisticFailTime"}->{"legend"} = '"RealServer Failtime"';
  $config->{graph}->{"snL4RealServerStatisticFailTime"}->{"units"} = '"Sec"';
  $config->{graph}->{"snL4RealServerStatisticFailTime"}->{"si-units"} = 'false';
  $config->{graph}->{"snL4RealServerStatisticFailTime"}->{"y-min"} = '0';
  $config->{graph}->{"snL4RealServerStatisticFailTime"}->{"y-max"} = '1000';

  $config->{graph}->{"snL4RealServerPortStatusRxPkts"}->{"color"} = 'dark-green';
  $config->{graph}->{"snL4RealServerPortStatusRxPkts"}->{"draw-as"} = 'AREA';
  $config->{graph}->{"snL4RealServerPortStatusRxPkts"}->{"y-axis"} = '"Packets"';
  $config->{graph}->{"snL4RealServerPortStatusRxPkts"}->{"legend"} = '"Packets received on the tcp/udp port of the RealServer"';
  $config->{graph}->{"snL4RealServerPortStatusRxPkts"}->{"units"} = '"Pkts"';
  $config->{graph}->{"snL4RealServerPortStatusRxPkts"}->{"si-units"} = 'false';
  $config->{graph}->{"snL4RealServerPortStatusRxPkts"}->{"y-min"} = '0';

  $config->{graph}->{"snL4RealServerPortStatusTxPkts"}->{"color"} = 'blue';
  $config->{graph}->{"snL4RealServerPortStatusTxPkts"}->{"y-axis"} = '"Packets"';
  $config->{graph}->{"snL4RealServerPortStatusTxPkts"}->{"legend"} = '"Packets transmit on the tcp/udp port of the RealServer"';
  $config->{graph}->{"snL4RealServerPortStatusTxPkts"}->{"units"} = '"Pkts"';
  $config->{graph}->{"snL4RealServerPortStatusTxPkts"}->{"si-units"} = 'false';
  $config->{graph}->{"snL4RealServerPortStatusTxPkts"}->{"y-min"} = '0';

  $config->{graph}->{"snL4RealServerPortStatusTotalConnection"}->{"color"} = 'dark-green';
  $config->{graph}->{"snL4RealServerPortStatusTotalConnection"}->{"draw-as"} = 'AREA';
  $config->{graph}->{"snL4RealServerPortStatusTotalConnection"}->{"y-axis"} = '"Connection"';
  $config->{graph}->{"snL4RealServerPortStatusTotalConnection"}->{"legend"} = '"Total connection on the tcp/udp port of the RealServer"';
  $config->{graph}->{"snL4RealServerPortStatusTotalConnection"}->{"units"} = '"Co"';
  $config->{graph}->{"snL4RealServerPortStatusTotalConnection"}->{"si-units"} = 'false';
  $config->{graph}->{"snL4RealServerPortStatusTotalConnection"}->{"y-min"} = '0';

  $config->{graph}->{"snL4RealServerPortStatusCurrentConnection"}->{"color"} = 'blue';
  $config->{graph}->{"snL4RealServerPortStatusCurrentConnection"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"snL4RealServerPortStatusCurrentConnection"}->{"y-axis"} = '"Connection"';
  $config->{graph}->{"snL4RealServerPortStatusCurrentConnection"}->{"legend"} = '"Current connection on the tcp/udp port of the RealServer"';
  $config->{graph}->{"snL4RealServerPortStatusCurrentConnection"}->{"units"} = '"Co"';
  $config->{graph}->{"snL4RealServerPortStatusCurrentConnection"}->{"si-units"} = 'false';
  $config->{graph}->{"snL4RealServerPortStatusCurrentConnection"}->{"y-min"} = '0';

  $config->{graph}->{"snL4RealServerPortStatusPeakConnection"}->{"color"} = 'red';
  $config->{graph}->{"snL4RealServerPortStatusPeakConnection"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"snL4RealServerPortStatusPeakConnection"}->{"y-axis"} = '"Connections"';
  $config->{graph}->{"snL4RealServerPortStatusPeakConnection"}->{"legend"} = '"Peak connection on the tcp/udp port of the RealServer"';
  $config->{graph}->{"snL4RealServerPortStatusPeakConnection"}->{"units"} = '"Co"';
  $config->{graph}->{"snL4RealServerPortStatusPeakConnection"}->{"si-units"} = 'false';
  $config->{graph}->{"snL4RealServerPortStatusPeakConnection"}->{"y-min"} = '0';

  $config->{graph}->{"snL4RealServerPortStatusRxBytes"}->{"color"} = 'dark-green';
  $config->{graph}->{"snL4RealServerPortStatusRxBytes"}->{"draw-as"} = 'AREA';
  $config->{graph}->{"snL4RealServerPortStatusRxBytes"}->{"y-axis"} = '"Bytes"';
  $config->{graph}->{"snL4RealServerPortStatusRxBytes"}->{"legend"} = '"Bytes received on the tcp/udp port of the RealServer"';
  $config->{graph}->{"snL4RealServerPortStatusRxBytes"}->{"units"} = '"Bytes"';
  $config->{graph}->{"snL4RealServerPortStatusRxBytes"}->{"si-units"} = 'false';
  $config->{graph}->{"snL4RealServerPortStatusRxBytes"}->{"y-min"} = '0';
  $config->{graph}->{"snL4RealServerPortStatusRxBytes"}->{"y-max"} = '100000000';

  $config->{graph}->{"snL4RealServerPortStatusTxBytes"}->{"color"} = 'blue';
  $config->{graph}->{"snL4RealServerPortStatusTxBytes"}->{"y-axis"} = '"Bytes"';
  $config->{graph}->{"snL4RealServerPortStatusTxBytes"}->{"legend"} = '"Bytes transmit on the tcp/udp port of the RealServer"';
  $config->{graph}->{"snL4RealServerPortStatusTxBytes"}->{"units"} = '"Bytes"';
  $config->{graph}->{"snL4RealServerPortStatusTxBytes"}->{"y-min"} = '0';
  $config->{graph}->{"snL4RealServerPortStatusTxBytes"}->{"y-max"} = '100000000';

  $config->{graph}->{"snL4RealServerPortStatusFailTime"}->{"color"} = 'dark-green';
  $config->{graph}->{"snL4RealServerPortStatusFailTime"}->{"draw-as"} = 'AREA';
  $config->{graph}->{"snL4RealServerPortStatusFailTime"}->{"y-axis"} = '"Seconds"';
  $config->{graph}->{"snL4RealServerPortStatusFailTime"}->{"legend"} = '"RealServer tcp/udp port Failtime"';
  $config->{graph}->{"snL4RealServerPortStatusFailTime"}->{"units"} = '"Sec"';
  $config->{graph}->{"snL4RealServerPortStatusFailTime"}->{"si-units"} = 'false';
  $config->{graph}->{"snL4RealServerPortStatusFailTime"}->{"y-min"} = '0';

  $config->{graph}->{"snL4VirtualServerStatisticReceivePkts"}->{"color"} = 'dark-green';
  $config->{graph}->{"snL4VirtualServerStatisticReceivePkts"}->{"draw-as"} = 'AREA';
  $config->{graph}->{"snL4VirtualServerStatisticReceivePkts"}->{"y-axis"} = '"Packets"';
  $config->{graph}->{"snL4VirtualServerStatisticReceivePkts"}->{"legend"} = '"Packets received by the VirtualServer"';
  $config->{graph}->{"snL4VirtualServerStatisticReceivePkts"}->{"units"} = '"Pkts"';
  $config->{graph}->{"snL4VirtualServerStatisticReceivePkts"}->{"si-units"} = 'false';
  $config->{graph}->{"snL4VirtualServerStatisticReceivePkts"}->{"y-min"} = '0';

  $config->{graph}->{"snL4VirtualServerStatisticTransmitPkts"}->{"color"} = 'blue';
  $config->{graph}->{"snL4VirtualServerStatisticTransmitPkts"}->{"y-axis"} = '"Packets"';
  $config->{graph}->{"snL4VirtualServerStatisticTransmitPkts"}->{"legend"} = '"Packets transmit by the VirtualServer"';
  $config->{graph}->{"snL4VirtualServerStatisticTransmitPkts"}->{"units"} = '"Pkts"';
  $config->{graph}->{"snL4VirtualServerStatisticTransmitPkts"}->{"si-units"} = 'false';
  $config->{graph}->{"snL4VirtualServerStatisticTransmitPkts"}->{"y-min"} = '0';

  $config->{graph}->{"snL4VirtualServerStatisticTotalConnections"}->{"color"} = 'dark-green';
  $config->{graph}->{"snL4VirtualServerStatisticTotalConnections"}->{"draw-as"} = 'AREA';
  $config->{graph}->{"snL4VirtualServerStatisticTotalConnections"}->{"y-axis"} = '"Connection"';
  $config->{graph}->{"snL4VirtualServerStatisticTotalConnections"}->{"legend"} = '"Total connection for the VirtualServer"';
  $config->{graph}->{"snL4VirtualServerStatisticTotalConnections"}->{"units"} = '"Co"';
  $config->{graph}->{"snL4VirtualServerStatisticTotalConnections"}->{"si-units"} = 'false';
  $config->{graph}->{"snL4VirtualServerStatisticTotalConnections"}->{"y-min"} = '0';

  $config->{graph}->{"snL4VirtualServerPortStatisticTotalConnection"}->{"color"} = 'dark-green';
  $config->{graph}->{"snL4VirtualServerPortStatisticTotalConnection"}->{"draw-as"} = 'AREA';
  $config->{graph}->{"snL4VirtualServerPortStatisticTotalConnection"}->{"y-axis"} = '"Connection"';
  $config->{graph}->{"snL4VirtualServerPortStatisticTotalConnection"}->{"legend"} = '"Total connection on the tcp/udp port of the VirtualServer"';
  $config->{graph}->{"snL4VirtualServerPortStatisticTotalConnection"}->{"units"} = '"Co"';
  $config->{graph}->{"snL4VirtualServerPortStatisticTotalConnection"}->{"si-units"} = 'false';
  $config->{graph}->{"snL4VirtualServerPortStatisticTotalConnection"}->{"y-min"} = '0';

  $config->{graph}->{"snL4VirtualServerPortStatisticCurrentConnection"}->{"color"} = 'blue';
  $config->{graph}->{"snL4VirtualServerPortStatisticCurrentConnection"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"snL4VirtualServerPortStatisticCurrentConnection"}->{"y-axis"} = '"Connection"';
  $config->{graph}->{"snL4VirtualServerPortStatisticCurrentConnection"}->{"legend"} = '"Current connection on the tcp/udp port of the VirtualServer"';
  $config->{graph}->{"snL4VirtualServerPortStatisticCurrentConnection"}->{"units"} = '"Co"';
  $config->{graph}->{"snL4VirtualServerPortStatisticCurrentConnection"}->{"si-units"} = 'false';
  $config->{graph}->{"snL4VirtualServerPortStatisticCurrentConnection"}->{"y-min"} = '0';

  $config->{graph}->{"snL4VirtualServerPortStatisticPeakConnection"}->{"color"} = 'red';
  $config->{graph}->{"snL4VirtualServerPortStatisticPeakConnection"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"snL4VirtualServerPortStatisticPeakConnection"}->{"y-axis"} = '"Connection"';
  $config->{graph}->{"snL4VirtualServerPortStatisticPeakConnection"}->{"legend"} = '"Peak connection on the tcp/udp port of the VirtualServer"';
  $config->{graph}->{"snL4VirtualServerPortStatisticPeakConnection"}->{"units"} = '"Co"';
  $config->{graph}->{"snL4VirtualServerPortStatisticPeakConnection"}->{"si-units"} = 'false';
  $config->{graph}->{"snL4VirtualServerPortStatisticPeakConnection"}->{"y-min"} = '0';

  foreach  $row (sort keys %{$result}) {
    my ($servername, $serverip, $adminstate) = ($result->{$row}->{2}, $result->{$row}->{3}, $result->{$row}->{4});
    next unless (defined $adminstate && $adminstate == 1);
    # $config->{target}->{$serverip}->{"router"} = "\"$hostname\"";
    # $config->{target}->{$serverip}->{"snmp-community"} = "\"$snmp_community\"";
    $config->{target}->{$serverip}->{"name"} = "\"$servername\"";
    $config->{target}->{$serverip}->{"long-desc"} = "\"Real Server $servername ($serverip)\"";
    $config->{target}->{$serverip}->{"short-desc"} = "\"Real Server $servername\"";
    $config->{target}->{$serverip}->{"target-type"} = "real_server";
    $reals{$servername} = $serverip;
  }
  
  $result = rehash (snmpUtils::snmpwalk ($snmp, $snL4VirtualServerTableOID));
  
  foreach  $row (sort keys %{$result}) {
    my ($servername, $serverip, $adminstate) = ($result->{$row}->{2}, $result->{$row}->{3}, $result->{$row}->{4});
    next unless (defined $adminstate && $adminstate == 1);
    # $config->{target}->{$serverip}->{"router"} = "\"$hostname\"";
    # $config->{target}->{$serverip}->{"snmp-community"} = "\"$snmp_community\"";
    $config->{target}->{$serverip}->{"name"} = "\"$servername\"";
    $config->{target}->{$serverip}->{"long-desc"} = "\"Virtual Server $servername ($serverip)\"";
    $config->{target}->{$serverip}->{"short-desc"} = "\"Virtual Server $servername\"";
    $config->{target}->{$serverip}->{"target-type"} = "virtual_server";
    $virtuals{$servername} = $serverip;
  }
  
  $result = rehash (snmpUtils::snmpwalk ($snmp, $snL4RealServerPortTableOID));
  
  foreach  $row (sort keys %{$result}) {
    my ($servername, $serverport, $adminstate) = ($result->{$row}->{2}, $result->{$row}->{3}, $result->{$row}->{4});
    my ($serverport1) = ($serverport == "65535")?"default":$serverport;
    next unless (defined $adminstate && $adminstate == 1);
    my $serverip = $reals{$servername};
    # $config->{target}->{$serverip . "." . $serverport}->{"router"} = "\"$hostname\"";
    # $config->{target}->{$serverip . "." . $serverport}->{"snmp-community"} = "\"$snmp_community\"";
    $config->{target}->{$serverip . "." . $serverport}->{"name"} = "\"$serverip\"";
    $config->{target}->{$serverip . "." . $serverport}->{"long-desc"} = "\"Real Server $servername ($serverip) Port $serverport1\"";
    $config->{target}->{$serverip . "." . $serverport}->{"short-desc"} = "\"Real Server $servername-$serverport1\"";
    $config->{target}->{$serverip . "." . $serverport}->{"target-type"} = "real_server_port";
  }
  
  $result = rehash (snmpUtils::snmpwalk ($snmp, $snL4VirtualServerPortTableOID));
  
  foreach  $row (sort keys %{$result}) {
    my ($servername, $serverport, $adminstate) = ($result->{$row}->{2}, $result->{$row}->{3}, $result->{$row}->{4});
    my ($serverport1) = ($serverport == "65535")?"default":$serverport;
    next unless (defined $adminstate && $adminstate == 1);
    my $serverip = $virtuals{$servername};
    # $config->{target}->{$serverip . "." . $serverport}->{"router"} = "\"$hostname\"";
    # $config->{target}->{$serverip . "." . $serverport}->{"snmp-community"} = "\"$snmp_community\"";
    $config->{target}->{$serverip . "." . $serverport}->{"name"} = "\"$serverip\"";
    $config->{target}->{$serverip . "." . $serverport}->{"long-desc"} = "\"Virtual Server $servername ($serverip) Port $serverport1\"";
    $config->{target}->{$serverip . "." . $serverport}->{"short-desc"} = "\"Virtual Server $servername-$serverport1\"";
    $config->{target}->{$serverip . "." . $serverport}->{"target-type"} = "virtual_server_port";
  }

  #CHIRP::listInterfaces::listInterfaces ($config, $hostname, $snmp_community, 1, 1, 0, 1, "bits", []);
  CHIRP::listInterfaces::listInterfaces ($config, $hostname, $snmp_community, $sup64bit, 1, 0, 1, "bits", []);

  $config->{target}->{"$hostname.CPU"}->{"long-desc"} = '"[CPU] Device CPU Utilization"';
  $config->{target}->{"$hostname.CPU"}->{"short-desc"} = '"CPU Utilization"';
  $config->{target}->{"$hostname.CPU"}->{"target-type"} = '"CPU"';
  
  return $config;
 
}

1; 

sub rehash (@) {
  my @a = @_;
  my $b = {};
  my $row;

  foreach $row (@a) {
    my ($index, $content) = split (/:/, $row, 2);
    my ($i, $j) = split (/\./, $index, 2);
    $b->{$j}->{$i} = $content;
  }

  return $b;
}

