# -*- perl -*-

use strict;

package CHIRP::Identify;

use lib "/usr/local/cricket/lib";
use snmpUtils;
use lib::Lowercase;
use lib::WriteConfig;
use Data::Dump qw (dump);

my $snmpCache  = {};

sub GenConfig ($$$$$$);
# hostname, community, snmp_port, config, sitefile

sub Identify ($$$$$$);
# hostname, community, snmp_port, config, debug
 
sub Match ($$$$$$$);
# config, device, hostname, community, snmp_port, match, debug

sub Identify ($$$$$$) {

  my($hostname, $community, $snmp_port, $config, $debug, $quiet) = @_;

  if ($debug) {
    print "Identify: $hostname, $community, $snmp_port, $config, $debug, $quiet\n";
  }

  my ($snmp) = "$community\@$hostname:$snmp_port";
  my ($snmp_cache) = {};
  my ($deviceTable) = {};

  # read in list of unknown devices, put in table
  foreach my $vendor_name (@{$config->{"vendor"}}) {
    my $vendor = $vendor_name->{"vname"}->[0];
    if ($vendor =~ /[Uu]nknown/) {
      if ($debug) {
        print "Vendor:$vendor\n";
      }
      foreach my $device_name (@{$vendor_name->{"device"}}) {
        my $device = $device_name->{"model"}->[0];
        foreach (keys %{$device_name}) {
          $deviceTable->{$_} = $device_name->{$_}->[0];
          $deviceTable->{$_} =~ s/^"//;
          $deviceTable->{$_} =~ s/"$//;
          if ($debug) {
            print "\tkey:  $_\n";
            print "\tvalue:  $$deviceTable{$_}\n";
          }
        }
      }
    }
  }


  VENDOR:
  foreach my $vendor_name (@{$config->{"vendor"}}) {
    my $vendor = $vendor_name->{"vname"}->[0];
    if ($debug) {
      print "Identify: vendor: $vendor\n"; 
    }
    next if ($vendor =~ /[Uu]nknown/);

    # DEVICE:
    foreach my $device_name (@{$vendor_name->{"device"}}) {
      my $device = $device_name->{"model"}->[0];
      if ($debug) {
        print "Identify: device: $device\n";
      }
      next if($device =~ /--default--/);

      # MATCH
      # get the <match>...</match>

      foreach my $exprline (@{$device_name->{"match"}}) {
        if ($debug) {
          print "Match expr = $exprline\n";
        }

        $exprline =~ s/^\n*//g;
        $exprline =~ s/\n*$//g;

        next if ($exprline =~ /^\s*$/);

        # this is where we "evaluate" the expression. currently supported is (tag1 & tag2 ...).
        # has to be replaced with more "elaborate" boolean expression evaluator.

        my (@tags) = split(/\s*\&\s*/, $exprline);

        if ($debug) {
          print "match tags: ", join (" " ,@tags), "\n";
        }

        # need to match all tags in @tags 
        # and || with the lines

        foreach my $tag (@tags) {
          if ($debug) {
            print "tag: $tag\n";
            print "trying to match $tag\n";
          }
  
          if(Match($config, $device, $hostname, $community, $snmp_port, $tag, $debug)) {
          if ($debug) {
            print "match **** : host:$hostname is $vendor, $device\n";
          }
          $deviceTable->{name} = $device;
          $deviceTable->{vendor} = $vendor;

          $deviceTable->{model} = $device_name->{"model"}->[0];
          $deviceTable->{module} = $device_name->{"module"}->[0];
          $deviceTable->{package} = $device_name->{"package_name"}->[0];

          if(!defined($device_name->{"options"}->[0])) {
            $deviceTable->{options} = "";
          } else {
            $deviceTable->{options} = $device_name->{"options"}->[0];
          }
          
          if ($debug) {
            print "Name:     $deviceTable->{name}\n";
            print "Vendor:   $deviceTable->{vendor}\n";
            print "Module:   $deviceTable->{module}\n";
            print "Options:  $deviceTable->{options}\n";
          }
          print "\tPackage: $deviceTable->{package}\n" unless ($quiet);
          print "\tModel:   $deviceTable->{model}\n" unless ($quiet);

          last VENDOR;
          }
        }
      }
    } # device 
  } # vendor 

  return $deviceTable;

} # Identify

sub Match ($$$$$$$) {

  my($config, $device, $hostname, $community, $snmp_port, $match, $debug) = @_;
  
  if ($debug) {
    print "MATCH: $config, $device, $hostname, $community, $snmp_port, $match\n";
  }

  my($snmpValue);

  if($match =~ /^snmp:\/\//) {
    $match =~ s/%snmp-host%/$hostname/i;
    $match =~ s/%snmp-community%/$community/i;
    $match =~ s/%snmp-port%/$snmp_port/i;
    $match =~ s/%snmp%/$community\@$hostname:$snmp_port/i;

    my($snmp, $value) = split(/\s*=\s*/, $match);
    my($oid_desc) = $snmp =~ /\/([^\/]*)$/;

    $snmp =~ s/snmp:\/\///;
    $snmp =~ s/\/[^\/]*$//;

    if ($debug) {
      print "MATCH: $snmp, $value, $oid_desc\n";
    }

    # match oid_name from id.xml
    my $oid;
    foreach my $oid_name (@{$config->{"oid"}}) {
      my $oidn = $oid_name->{"oid_name"}->[0];

      if (lc($oid_desc) eq lc($oidn)) {
        $oid = $oid_name->{"oid_value"}->[0];
        if ($debug) {
          print "MATCH: $oid, $oidn\n";
        }
      }
    }

    if ($debug) {
      print "oid:$oid\n";
    }
    
    if(!defined($oid)) {
      print STDERR "ERROR: Device \"$device\" has an invalid OID: >$match<\n";
      return undef;
    } elsif (defined($snmpCache->{lc("$hostname.$oid")})) {
      $snmpValue = $snmpCache->{lc("$hostname.$oid")};
    } else {
      ($snmpValue) = snmpUtils::get($snmp, $oid);
      if(defined($snmpValue)) {
        $snmpCache->{lc("$hostname.$oid")} = $snmpValue;
      } else {
        $snmpCache->{lc("$hostname.$oid")} = "";
      }
    }
    if ($debug) {
      print "snmpValue:$snmpValue, value:$value\n";
    }
    if(defined($snmpValue)) {
      if ($debug) {
        print "return ", ($snmpValue =~ /$value/), "\n";;
      }
      return 1 if($snmpValue =~ /$value/);
    } else {
      print STDERR "WARNING: Unable to retrieve oid >$oid< from $hostname\n";
    }
  } else {
    print STDERR "ERROR: Device \"$device\" has an invalid match tag: >$match<\n";
  }
  return 0;
}

1;


sub GenConfig ($$$$$$) {

  my($hostname, $community, $snmp_port, $devConfig, $debug, $quiet) = @_;

  if ($debug) {
    print "GenConfig: $hostname, $community, $snmp_port, $devConfig\n";
  }

  my ($config);
  my ($deviceTable) = {};

  if(!defined($hostname)) {
    print STDERR "ERROR: Hostname must be specified in call to CHIRP::GenConfig\n";
    return undef;
  }

  $community = "public" unless defined($community);
  $snmp_port = "161" unless defined ($snmp_port);

  $deviceTable = CHIRP::Identify::Identify($hostname, $community, $snmp_port, $devConfig, $debug, $quiet);

  if(!defined($deviceTable) || !defined($deviceTable->{module}) || !defined($deviceTable->{package}) || 
     !defined($deviceTable->{options})) {
    print STDERR "ERROR: lib::CHIRP::Identify failed to return valid device table.\n";
    return undef;
  }

  eval " use $deviceTable->{module}; ";

  if($@) {
    print STDERR "ERROR: Failed to load module $deviceTable->{module}\nERROR: Error was " . $@ . "\n" if $@;
    return undef;
  }

 
  eval " \$config = " . $deviceTable->{package} . "::GenConfig(\$hostname, \$community, \$snmp_port, \$deviceTable->{options});"; 


  print STDERR $@ if $@;


  if(!defined(CHIRP::Lowercase::LowercaseConfig(\$config))) {
    print STDERR "ERROR: Failed to properly lowercase configuration\n";
    return undef;
  }

  if ($debug) {
    print "config returned\n";
  }
  return $config;
}

1;
