# -*- perl -*-

use strict;

package CHIRP::Juniper::MSeries;

use lib "/usr/local/cricket/lib";
use lib::listInterfaces;
use snmpUtils;

sub rehash (@) {
  my @a = @_;
  my $b = {};
  my $row;

  foreach $row (@a) {
    my ($index, $content) = split (/:/, $row, 2);
    my ($i, $j) = split (/\./, $index, 2);
    $b->{$j}->{$i} = $content;
  }

  return $b;
}

my $jnxOperatingDescr = "1.3.6.1.4.1.2636.3.1.13.1.5";
my $sysNameOID = ".1.3.6.1.2.1.1.5.0";

sub GenConfig {
  my($hostname, $snmp_community, $snmp_port, $options) = @_;

  $snmp_community = 'public' unless defined($snmp_community);
  $snmp_port = '161' unless defined($snmp_port);

  my($snmp) = "$snmp_community\@$hostname:$snmp_port";
  my($config) = {};

  my ($sysname) = snmpUtils::snmpget ($snmp, $sysNameOID);
  my @colors = ("dark-blue", "dark-green", "yellow", "red", "green", "blue", "black", "magenta", "cyan");

  $config->{target}->{"--default--"}->{"rrd-datafile"} =        '%dataDir%/%auto-target-name%.rrd';
  $config->{target}->{"--default--"}->{"router"} =            '%auto-target-name%';
  $config->{target}->{"--default--"}->{"router"} =              "\"$hostname\"";
  $config->{target}->{"--default--"}->{"snmp-host"} =           '%router%';
  $config->{target}->{"--default--"}->{"snmp-community"} = "$snmp_community";
  $config->{target}->{"--default--"}->{"snmp-port"} = "$snmp_port";
  $config->{target}->{"--default--"}->{"target-type"} =         'undef';
  $config->{target}->{"--default--"}->{"snmp-version"} =        '2c';
  $config->{target}->{"--default--"}->{"directory-desc"} =      "\"[Juniper M-Series] Statistics for $hostname ($sysname)\"";
  my $row;

  my %jnxModules;

  foreach $row (snmpUtils::snmpwalk ($snmp, $jnxOperatingDescr)) {
    my ($index, $content) = split (/:/, $row, 2);
    if ($content =~ /^SSB \d+/ || $content =~ /^FPC/ || $content =~ /^Routing Engine/) {
      $jnxModules{$index} = $content;
    }
  }
  my $dev_cnt = 0;

  ############# All Temperature ###############

  my %data;
  my $junEnvMonName = '1.3.6.1.4.1.2636.3.1.13.1.5'; # returns names of env mon sensors
  my $junEnvMon = "1.3.6.1.4.1.2636.3.1.13.1.7";

  foreach (snmpUtils::walk ($snmp, $junEnvMonName)) 
  {
    my ($oid, $value) = split (/:/, $_, 2);
    #backplane
    if ($value =~ /backplane/)
    {
      my $cod = "Backplane";
      my $rec = "Backplane";
      $data{$cod}{$rec}{desc} = "Backplane";
      $data{$cod}{$rec}{type} = "Backplane";
      $data{$cod}{$rec}{oid} = $oid;
      $data{$cod}{$rec}{desc2} = "Backplane";
    } #end of backplane
    #Power Supply X
    if ($value =~ /Power Supply (\w+)/)
    {
      my $cod = "Power_Supply";
      my $rec = "Power_Supply_$1";
      $data{$cod}{$rec}{desc} = "Power Supply " . $1;
      $data{$cod}{$rec}{type} = "Power_Supply_$1";
      $data{$cod}{$rec}{oid} = $oid;
      $data{$cod}{$rec}{desc2} = "$1";
    } #end of Power Supply X
    #Routing Engine X
    if (($value =~ /Routing Engine (\d+)/)&&($value !~ /Routing Engine \d+ PCMCIA/))
    {
      my $cod = "Routing_Engine";
      my $rec = "Routing_Engine_$1";
      $data{$cod}{$rec}{desc} = "Routing Engine " . $1;
      $data{$cod}{$rec}{type} = "Routing_Engine_$1";
      $data{$cod}{$rec}{oid} = $oid;
      $data{$cod}{$rec}{desc2} = "$1";
    } #end of Routing Engine X
    #SSB X Internet Processor II
    if ($value =~ /SSB (\d+) Internet/)
    {
      my $cod = "SSB";
      my $rec = "SSB_$1";
      $data{$cod}{$rec}{desc} = "SSB " . $1;
      $data{$cod}{$rec}{type} = "SSB_$1";
      $data{$cod}{$rec}{oid} = $oid;
      $data{$cod}{$rec}{desc2} = "$1";
    } #end of SSB X Internet Processor II
    #FPC @ 0/*/*
    if ($value =~ /FPC @ (\S+)/)
    {
      my $cod = "FPC";
      my $rec = "FPC_$1";
      $data{$cod}{$rec}{desc} = "FPC " . $1;
      $data{$cod}{$rec}{type} = "FPC_$1";
      $data{$cod}{$rec}{oid} = $oid;
      $data{$cod}{$rec}{desc2} = "$1";
    } #end of FPC @ 0/*/*
    #PIC: 1x G/E, 1000 BASE-SX @ 0/0/*
    if ($value =~ /PIC:/)
    {
      my ($t1) = ($value =~ /@ (\S+)/);
      my $cod = "PIC";
      my $rec = "FPC_$t1";
      $data{$cod}{$rec}{desc} = "PIC " . $t1;
      $data{$cod}{$rec}{type} = "FPC_$t1";
      $data{$cod}{$rec}{oid} = $oid;
      $data{$cod}{$rec}{desc2} = "$t1";
    } #end of PIC: 1x G/E, 1000 BASE-SX @ 0/0/*

  }  

  my $temp_g;
  my $temp_r;
  my $temp_d;
  for $temp_g ( keys %data ) 
  {
    my $gr1 = "";
    my $gr2 = "";
    for $temp_r ( keys %{ $data{$temp_g} } ) 
    {
      $config->{datasource}->{$data{$temp_g}{$temp_r}{type}}->{"ds-source"} = 
		"snmp://%snmp%/$junEnvMon.$data{$temp_g}{$temp_r}{oid}";
      $config->{datasource}->{$data{$temp_g}{$temp_r}{type}}->{"desc"} = 
		"Temperature on $data{$temp_g}{$temp_r}{desc}";
      $config->{datasource}->{$data{$temp_g}{$temp_r}{type}}->{"rrd-ds-type"} = "GAUGE";

      $config->{graph}->{$data{$temp_g}{$temp_r}{type}}->{"units"} = '"deg C"';
      $config->{graph}->{$data{$temp_g}{$temp_r}{type}}->{"draw-as"} = '"LINE2"';
      $config->{graph}->{$data{$temp_g}{$temp_r}{type}}->{"legend"} = "$data{$temp_g}{$temp_r}{desc2}";
      $config->{graph}->{$data{$temp_g}{$temp_r}{type}}->{"y-axis"} = '"Temperature deg C"';
      $config->{graph}->{$data{$temp_g}{$temp_r}{type}}->{"y-min"} = '"0"';

      $gr1 = $gr1 . $data{$temp_g}{$temp_r}{type} . ", ";
      $gr2 = $gr2 . $data{$temp_g}{$temp_r}{type} . " ";
    }
    $gr1 = substr($gr1,0,-2);
    $gr2 = substr($gr2,0,-1);

    $config->{targettype}->{"Temperature" . "_" . $temp_g}->{"ds"} = "$gr1";
    $config->{targettype}->{"Temperature" . "_" . $temp_g}->{"View"} = "Temp: $gr2";

    $config->{target}->{"Temperature" . "_" . $temp_g}->{"target-type"} = "Temperature" . "_" . $temp_g;
    $config->{target}->{"Temperature" . "_" . $temp_g}->{"tempname"} = "$temp_g";
    $config->{target}->{"Temperature" . "_" . $temp_g}->{"long-desc"}= "Temperature statistics for $temp_g";
    $config->{target}->{"Temperature" . "_" . $temp_g}->{"short-desc"}= "Temperature for $temp_g";
  }
  ################  End of Temperature ############################

  $config->{datasource}->{"jnxCPU"}->{"rrd-ds-type"} = 'GAUGE';
  $config->{datasource}->{"jnxCPU"}->{"ds-source"} = 'snmp://%snmp%/.1.3.6.1.4.1.2636.3.1.13.1.8.%inst%';

  $config->{datasource}->{"jnxHEAP"}->{"rrd-ds-type"} = 'GAUGE';
  $config->{datasource}->{"jnxHEAP"}->{"ds-source"} = 'snmp://%snmp%/.1.3.6.1.4.1.2636.3.1.13.1.11.%inst%';

  $config->{datasource}->{"jnxBUFF"}->{"rrd-ds-type"} = 'GAUGE';
  $config->{datasource}->{"jnxBUFF"}->{"ds-source"} = 'snmp://%snmp%/.1.3.6.1.4.1.2636.3.1.13.1.12.%inst%';

  $config->{graph}->{"jnxHEAP"}->{"color"} = "blue";
  $config->{graph}->{"jnxHEAP"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"jnxHEAP"}->{"y-axis"} = 'Percent';
  $config->{graph}->{"jnxHEAP"}->{"legend"} = '"Heap Utilization"';
  $config->{graph}->{"jnxHEAP"}->{"units"} = '"%"';
  $config->{graph}->{"jnxHEAP"}->{"si-units"} = 'false';
  $config->{graph}->{"jnxHEAP"}->{"y-min"} = '0';
  $config->{graph}->{"jnxHEAP"}->{"y-max"} = '100';

  $config->{graph}->{"jnxBUFF"}->{"color"} = "green";
  $config->{graph}->{"jnxBUFF"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"jnxBUFF"}->{"y-axis"} = 'Percent';
  $config->{graph}->{"jnxBUFF"}->{"legend"} = '"Buffer Utilization"';
  $config->{graph}->{"jnxBUFF"}->{"units"} = '"%"';
  $config->{graph}->{"jnxBUFF"}->{"si-units"} = 'false';
  $config->{graph}->{"jnxBUFF"}->{"y-min"} = '0';
  $config->{graph}->{"jnxBUFF"}->{"y-max"} = '100';

  $config->{graph}->{"jnxCPU"}->{"color"} = "red";
  $config->{graph}->{"jnxCPU"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"jnxCPU"}->{"y-axis"} = 'Percent';
  $config->{graph}->{"jnxCPU"}->{"legend"} = '"CPU Utilization for"';
  $config->{graph}->{"jnxCPU"}->{"units"} = '"%"';
  $config->{graph}->{"jnxCPU"}->{"si-units"} = 'false';
  $config->{graph}->{"jnxCPU"}->{"y-min"} = '0';
  $config->{graph}->{"jnxCPU"}->{"y-max"} = '100';

  $config->{oid}->{"jnxOperatingDescr"} = "$jnxOperatingDescr";
  $config->{map}->{"module-name"}->{"base-oid"} = "jnxOperatingDescr";
  $config->{map}->{"module-name"}->{"match"} = "%module-id%";

  $config->{targettype}->{"jnxChassis"}->{"ds"} = '"jnxCPU, jnxHEAP, jnxBUFF"';
  $config->{targettype}->{"jnxChassis"}->{"view"} = '"Utilizations: jnxCPU jnxHEAP jnxBUFF"';

  my @jnxDSCPU = map { 
    my $a = $_;
    s/\./_/g; 

    $config->{target}->{"$hostname.$_"}->{"target-type"} = '"jnxChassis"';
    $config->{target}->{"$hostname.$_"}->{"long-desc"} = '"Chassis Utilization for ' . $jnxModules{$a} . '"';
    $config->{target}->{"$hostname.$_"}->{"short-desc"} = '"Utilization for ' . $jnxModules{$a} . '"';
    $config->{target}->{"$hostname.$_"}->{"module-id"} = '"' . $jnxModules{$a} . '"';
    $config->{target}->{"$hostname.$_"}->{"inst"} = 'map(module-name)';

    "jnxCPU$_";
  } keys %jnxModules;

  CHIRP::listInterfaces::listInterfaces ($config, $hostname, $snmp_community, 1, 1, 0, 0, "bits", 
                         ["fxp", "tap", "pimd", "pime", "ipip", '^gre$', "lo0", '\.0$' ]);

  return $config;
}

1;

__END__
# -*- perl -*-

