# -*- perl -*-

use strict;

package CHIRP::Netscaler::9000;

use lib "/usr/local/cricket/lib";
use lib::listInterfaces;
use snmpUtils;

my $sysName = ".1.3.6.1.2.1.1.5.0";

sub GenConfig {
  my($hostname, $snmp_community, $snmp_port, $options) = @_;

  $snmp_community = 'public' unless defined($snmp_community);
  $snmp_port = '161' unless defined($snmp_port);

  my($snmp) = "$snmp_community\@$hostname:$snmp_port";
  my($config) = {};

  my ($sysname) = snmpUtils::snmpget ($snmp, $sysName);

  ######################## interface info #############################
  my $int_name = '1.3.6.1.4.1.5951.1.2.1.19.1.2';
  my $int_media = '1.3.6.1.4.1.5951.1.2.1.19.1.3';

  my $ifAdminStatOID = '1.3.6.1.2.1.2.2.1.7';
  my $ifOperStatOID  = '1.3.6.1.2.1.2.2.1.8';
  my $ifSpeedOID     = '1.3.6.1.2.1.2.2.1.5';

  $config->{oid}->{"intRxBw"} = '.1.3.6.1.4.1.5951.1.2.1.19.1.4';
  $config->{oid}->{"intTxBw"} = '.1.3.6.1.4.1.5951.1.2.1.19.1.11';

  $config->{oid}->{"intRxFrameE"} = '.1.3.6.1.4.1.5951.1.2.1.19.1.8';
  $config->{oid}->{"intRxCrcE"} = '.1.3.6.1.4.1.5951.1.2.1.19.1.9';
  $config->{oid}->{"intRxAlignE"} = '.1.3.6.1.4.1.5951.1.2.1.19.1.10';

  $config->{oid}->{"intTXCollE"} = '.1.3.6.1.4.1.5951.1.2.1.19.1.17';

  $config->{datasource}->{if_rx_bw}->{"ds-source"} =  'snmp://%snmp%/intRxBw.%inst%';
  $config->{datasource}->{if_rx_bw}->{"rrd-ds-type"} = 'GAUGE';

  $config->{datasource}->{if_tx_bw}->{"ds-source"} =  'snmp://%snmp%/intTxBw.%inst%';
  $config->{datasource}->{if_tx_bw}->{"rrd-ds-type"} = 'GAUGE';

  $config->{datasource}->{if_rx_frame_err}->{"ds-source"} =  'snmp://%snmp%/intRxFrameE.%inst%';
  $config->{datasource}->{if_rx_frame_err}->{"rrd-ds-type"} = 'counter';
  
  $config->{datasource}->{if_rx_crc_err}->{"ds-source"} =  'snmp://%snmp%/intRxCrcE.%inst%';
  $config->{datasource}->{if_rx_crc_err}->{"rrd-ds-type"} = 'counter';
  
  $config->{datasource}->{if_rx_align_err}->{"ds-source"} =  'snmp://%snmp%/intRxAlignE.%inst%';
  $config->{datasource}->{if_rx_align_err}->{"rrd-ds-type"} = 'counter';

  $config->{datasource}->{if_tx_coll_err}->{"ds-source"} =  'snmp://%snmp%/intTXCollE.%inst%';
  $config->{datasource}->{if_tx_coll_err}->{"rrd-ds-type"} = 'counter';

  $config->{graph}->{"if_rx_bw"}->{"color"} = 'dark-green';
  $config->{graph}->{"if_rx_bw"}->{"draw-as"} = 'AREA';
  $config->{graph}->{"if_rx_bw"}->{"legend"} = '"Average bits in"';
  $config->{graph}->{"if_rx_bw"}->{"y-axis"} = '"bits per second"';
  $config->{graph}->{"if_rx_bw"}->{"units"} = '"bits/sec"';

  $config->{graph}->{"if_tx_bw"}->{"color"} = 'blue';
  $config->{graph}->{"if_tx_bw"}->{"draw-as"} = 'LINE1';
  $config->{graph}->{"if_tx_bw"}->{"legend"} = '"Average bits out"';
  $config->{graph}->{"if_tx_bw"}->{"y-axis"} = '"bits per second"';
  $config->{graph}->{"if_tx_bw"}->{"units"} = '"bits/sec"';

  $config->{graph}->{"if_rx_frame_err"}->{"color"} = 'dark-green';
  $config->{graph}->{"if_rx_frame_err"}->{"draw-as"} = 'LINE1';
  $config->{graph}->{"if_rx_frame_err"}->{"legend"} = '"Frame Errors"';
  $config->{graph}->{"if_rx_frame_err"}->{"y-axis"} = '"Errors"';
  $config->{graph}->{"if_rx_frame_err"}->{"units"} = '"Errors"';

  $config->{graph}->{"if_rx_crc_err"}->{"color"} = 'blue';
  $config->{graph}->{"if_rx_crc_err"}->{"draw-as"} = 'LINE1';
  $config->{graph}->{"if_rx_crc_err"}->{"legend"} = '"CRC Errors"';
  $config->{graph}->{"if_rx_crc_err"}->{"y-axis"} = '"Errors"';
  $config->{graph}->{"if_rx_crc_err"}->{"units"} = '"Errors"';

  $config->{graph}->{"if_rx_align_err"}->{"color"} = 'red';
  $config->{graph}->{"if_rx_align_err"}->{"draw-as"} = 'LINE1';
  $config->{graph}->{"if_rx_align_err"}->{"legend"} = '"Alignment Errors"';
  $config->{graph}->{"if_rx_align_err"}->{"y-axis"} = '"Errors"';
  $config->{graph}->{"if_rx_align_err"}->{"units"} = '"Errors"';

  $config->{graph}->{"if_tx_coll_err"}->{"color"} = 'dark-green';
  $config->{graph}->{"if_tx_coll_err"}->{"draw-as"} = 'LINE1';
  $config->{graph}->{"if_tx_coll_err"}->{"legend"} = '"Collision Errors"';
  $config->{graph}->{"if_tx_coll_err"}->{"y-axis"} = '"Errors"';
  $config->{graph}->{"if_tx_coll_err"}->{"units"} = '"Errors"';


  $config->{targettype}->{"standard-interface"}->{"ds"} = '"if_rx_bw, if_tx_bw, if_rx_frame_err, if_rx_crc_err, if_rx_align_err, if_tx_coll_err"';
  $config->{targettype}->{"standard-interface"}->{"view"} = '"Bandwidth: if_rx_bw if_tx_bw, RX Err:  if_rx_frame_err if_rx_crc_err if_rx_align_err, TX Err:  if_tx_coll_err"';
  $config->{targettype}->{"standard-interface"}->{"snmp-version"} = "1";

  ROW:
  foreach (snmpUtils::walk ($snmp, $int_media))
  {
    my ($oid, $value) = split (/:/, $_, 2);
    my ($adminstat) = snmpUtils::get($snmp, "$ifAdminStatOID.$oid");
    my ($operstat) = snmpUtils::get($snmp, "$ifOperStatOID.$oid");
    my ($if_name) = snmpUtils::get($snmp, "$int_name.$oid");
    my ($ifSpeed) = snmpUtils::get($snmp, "$ifSpeedOID.$oid");
    
    next ROW if ($adminstat != 1 || $operstat != 1);

    my $real_ifname = $if_name;

    # take out any / and replace with _
    $if_name =~ s/[\/\s:]/\_/g;
    
    $config->{target}->{$if_name}->{"interface-name"} = "$real_ifname"; 
    $config->{target}->{$if_name}->{"short-desc"} = "$value";
    $config->{target}->{$if_name}->{"if-speed"} = "$ifSpeed";
    $config->{target}->{$if_name}->{"inst"} = 'map(interface-name)';
    $config->{target}->{$if_name}->{"target-type"} = "standard-interface";
  }      

  ######################## vserver info #############################

  my $vserver_oid = '1.3.6.1.4.1.5951.4.1.3.1.1.1';
  my $vsvrCurClntConnections = '1.3.6.1.4.1.5951.4.1.3.1.1.7';
  my $vsvrCurSrvrConnections = '1.3.6.1.4.1.5951.4.1.3.1.1.7';

  foreach (snmpUtils::walk ($snmp, $vserver_oid))
  {
    my ($oid, $value) = split (/:/, $_, 2);

    #vsvrCurClntConnections
    my $cl_oid = $vsvrCurClntConnections . "." . $oid;
    my $sv_oid = $vsvrCurSrvrConnections . "." . $oid;

    my $cl_desc = $value . "_vsvrCurClntConnections";
    my $sv_desc = $value . "_vsvrCurSrvrConnections";

    $config->{datasource}->{$cl_desc}->{"ds-source"} =  "snmp://%snmp%/$cl_oid";
    $config->{datasource}->{$cl_desc}->{"desc"} = "Current Client Connections to $value";
    $config->{datasource}->{$cl_desc}->{"rrd-ds-type"} = "GAUGE";

    $config->{datasource}->{$sv_desc}->{"ds-source"} =  "snmp://%snmp%/$sv_oid";
    $config->{datasource}->{$sv_desc}->{"desc"} = "Connections to real servers behind $value";
    $config->{datasource}->{$sv_desc}->{"rrd-ds-type"} = "GAUGE";

    $config->{graph}->{$cl_desc}->{"units"} = '"connections"';
    $config->{graph}->{$cl_desc}->{"draw-as"} = '"LINE2"';
    $config->{graph}->{$cl_desc}->{"legend"} = "client connections";
    $config->{graph}->{$cl_desc}->{"y-axis"} = '"# of connections"';
    $config->{graph}->{$cl_desc}->{"y-min"} = '"0"';

    $config->{graph}->{$sv_desc}->{"units"} = '"connections"';
    $config->{graph}->{$sv_desc}->{"draw-as"} = '"LINE2"';
    $config->{graph}->{$sv_desc}->{"legend"} = "server connections";
    $config->{graph}->{$sv_desc}->{"y-axis"} = '"# of connections"';
    $config->{graph}->{$sv_desc}->{"y-min"} = '"0"';

    my $name = "vserver_connections_" . $value;

    $config->{targettype}->{$name}->{"ds"} = "$cl_desc, $sv_desc";
    $config->{targettype}->{$name}->{"View"} = "Vserver Connections: $cl_desc $sv_desc";
    
    $config->{target}->{$name}->{"target-type"} = "vserver_connections_$value";
    #$config->{target}->{$name}->{"tempname"} = "mike";
    $config->{target}->{$name}->{"long-desc"}= "Current Client/Server Connections for $value";
    $config->{target}->{$name}->{"short-desc"}= "Current Client/Server Connections to Vserver";

  }

  ####################### end of vserver info #######################
  ####################### real server info ##########################

  my $rsrvr_oid = '1.3.6.1.4.1.5951.4.1.2.1.1.1';
  my $rsrvrEstConnections_oid = '1.3.6.1.4.1.5951.4.1.2.1.1.8';

  RSVR:
  foreach (snmpUtils::walk ($snmp, $rsrvr_oid))
  {
    my ($oid, $value) = split (/:/, $_, 2);
    #print "RSRVR OID = $oid\n";
    #print "RSRVR VALUE = $value\n";

    my $rsrvr_est_conn_oid = $rsrvrEstConnections_oid . "." . $oid;

    my $desc = $value . "_rsvrEstConnections";

    $config->{datasource}->{$desc}->{"ds-source"} =  "snmp://%snmp%/$rsrvr_est_conn_oid";
    $config->{datasource}->{$desc}->{"desc"} = "Established Connections to $value";
    $config->{datasource}->{$desc}->{"rrd-ds-type"} = "GAUGE";

    $config->{graph}->{$desc}->{"units"} = '"connections"';
    $config->{graph}->{$desc}->{"draw-as"} = '"LINE2"';
    $config->{graph}->{$desc}->{"legend"} = "est connections";
    $config->{graph}->{$desc}->{"y-axis"} = '"# of connections"';
    $config->{graph}->{$desc}->{"y-min"} = '"0"';

    my $name = "rserver_connections_" . $value;

    $config->{targettype}->{$name}->{"ds"} = "$desc";
    $config->{targettype}->{$name}->{"View"} = "Real Server Connections: $desc";

    $config->{target}->{$name}->{"target-type"} = "rserver_connections_$value";
    $config->{target}->{$name}->{"long-desc"}= "Established Connections for $value";
    $config->{target}->{$name}->{"short-desc"}= "Established Connections to Real Server";
  }

  ####################### end of real server info ###################
  
  my $row;
  
  $config->{target}->{"--default--"}->{"rrd-datafile"} =	'%dataDir%/%auto-target-name%.rrd';
  $config->{target}->{"--default--"}->{"router"} =	 	"\"$hostname\"";
  $config->{target}->{"--default--"}->{"snmp-host"} =	 	'%router%';
  $config->{target}->{"--default--"}->{"snmp-community"} = "$snmp_community";
  $config->{target}->{"--default--"}->{"snmp-port"} = "$snmp_port";
  $config->{target}->{"--default--"}->{"target-type"} =	 	'undef';
  $config->{target}->{"--default--"}->{"snmp-version"} = 	'1';
  $config->{target}->{"--default--"}->{"directory-desc"} =	"\"[Netscaler 9000] Statistics for $hostname \"";

  $config->{oid}->{"cpuUsage"}			= '.1.3.6.1.4.1.5951.1.2.1.14.0';
  $config->{oid}->{"curClientConnections"}	= '.1.3.6.1.4.1.5951.1.2.1.2.0';
  $config->{oid}->{"curServerConnections"}	= '.1.3.6.1.4.1.5951.1.2.1.4.0';
  $config->{oid}->{"curClientEstConn"}		= '.1.3.6.1.4.1.5951.1.2.1.16.0';
  $config->{oid}->{"curServerEstConn"}		= '.1.3.6.1.4.1.5951.1.2.1.17.0';
  $config->{oid}->{"curUdpSessions"}		= '.1.3.6.1.4.1.5951.1.2.1.20.2.0';
  $config->{oid}->{"clientConnRefused"}		= '.1.3.6.1.4.1.5951.1.2.1.5.0';

  $config->{datasource}->{"--default--"}->{"rrd-ds-type"} 	= "GAUGE";
  $config->{datasource}->{"--default--"}->{"rrd-heartbeat"} 	= "1800";
  $config->{datasource}->{"--default--"}->{"rrd-min"}		= "undef";
  $config->{datasource}->{"--default--"}->{"rrd-max"}		= "undef";

  $config->{datasource}->{"cpuUsage"}->{"ds-source"} = 'snmp://%snmp%/cpuUsage';
  $config->{datasource}->{"curClientConnections"}->{"ds-source"} = 'snmp://%snmp%/curClientConnections';
  $config->{datasource}->{"curServerConnections"}->{"ds-source"} = 'snmp://%snmp%/curServerConnections';
  $config->{datasource}->{"curClientEstConn"}->{"ds-source"} = 'snmp://%snmp%/curClientEstConn';
  $config->{datasource}->{"curServerEstConn"}->{"ds-source"} = 'snmp://%snmp%/curServerEstConn';
  $config->{datasource}->{"curUdpSessions"}->{"ds-source"} = 'snmp://%snmp%/curUdpSessions';
  $config->{datasource}->{"clientConnRefused"}->{"ds-source"} = 'snmp://%snmp%/clientConnRefused';

  $config->{targettype}->{"CPU"}->{"ds"} = '"cpuUsage"';
  $config->{targettype}->{"CPU"}->{"view"} = '"CPU Utilization: cpuUsage"';

  $config->{targettype}->{"Connections"}->{"ds"} = '"curClientConnections, curServerConnections"';
  $config->{targettype}->{"Connections"}->{"view"} = '"Connections: curClientConnections curServerConnections"';

  $config->{targettype}->{"EstConnections"}->{"ds"} = '"curClientEstConn, curServerEstConn"';
  $config->{targettype}->{"EstConnections"}->{"view"} = '"EstConnections: curClientEstConn curServerEstConn"';

  $config->{targettype}->{"UDPConnections"}->{"ds"} = '"curUdpSessions"';
  $config->{targettype}->{"UDPConnections"}->{"view"} = '"UDPConnections: curUdpSessions"';

  $config->{targettype}->{"clientConnRefused"}->{"ds"} = '"clientConnRefused"';
  $config->{targettype}->{"clientConnRefused"}->{"view"} = '"RefusedConn: clientConnRefused"';


  $config->{graph}->{"cpuUsage"}->{"color"} = 'dark-green';
  $config->{graph}->{"cpuUsage"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"cpuUsage"}->{"y-axis"} = '"Percent"';
  $config->{graph}->{"cpuUsage"}->{"legend"} = '"CPU"';
  $config->{graph}->{"cpuUsage"}->{"units"} = '"%"';
  $config->{graph}->{"cpuUsage"}->{"si-units"} = 'false';
  $config->{graph}->{"cpuUsage"}->{"y-min"} = '0';

  $config->{graph}->{"curClientConnections"}->{"color"} = 'dark-green';
  $config->{graph}->{"curClientConnections"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"curClientConnections"}->{"y-axis"} = '"# of Connections"';
  $config->{graph}->{"curClientConnections"}->{"legend"} = '"Client Connections"';
  $config->{graph}->{"curClientConnections"}->{"units"} = '"Connections"';
  $config->{graph}->{"curClientConnections"}->{"y-min"} = '0';

  $config->{graph}->{"curServerConnections"}->{"color"} = 'blue';
  $config->{graph}->{"curServerConnections"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"curServerConnections"}->{"y-axis"} = '"# of Connections"';
  $config->{graph}->{"curServerConnections"}->{"legend"} = '"Server Connections"';
  $config->{graph}->{"curServerConnections"}->{"units"} = '"Connections"';
  $config->{graph}->{"curServerConnections"}->{"y-min"} = '0';
 
  $config->{graph}->{"curClientEstConn"}->{"color"} = 'dark-green';
  $config->{graph}->{"curClientEstConn"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"curClientEstConn"}->{"y-axis"} = '"# of Connections"';
  $config->{graph}->{"curClientEstConn"}->{"legend"} = '"Client Established Connections"';
  $config->{graph}->{"curClientEstConn"}->{"units"} = '"Connections"';
  $config->{graph}->{"curClientEstConn"}->{"y-min"} = '0';

  $config->{graph}->{"curServerEstConn"}->{"color"} = 'blue';
  $config->{graph}->{"curServerEstConn"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"curServerEstConn"}->{"y-axis"} = '"# of Connections"';
  $config->{graph}->{"curServerEstConn"}->{"legend"} = '"Server Established Connections"';
  $config->{graph}->{"curServerEstConn"}->{"units"} = '"Connections"';
  $config->{graph}->{"curServerEstConn"}->{"y-min"} = '0';

  $config->{graph}->{"curUdpSessions"}->{"color"} = 'dark-green';
  $config->{graph}->{"curUdpSessions"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"curUdpSessions"}->{"y-axis"} = '"# of UDP Connections"';
  $config->{graph}->{"curUdpSessions"}->{"legend"} = '"UDP Connections"';
  $config->{graph}->{"curUdpSessions"}->{"units"} = '"Connections"';
  $config->{graph}->{"curUdpSessions"}->{"y-min"} = '0';

  $config->{graph}->{"clientConnRefused"}->{"color"} = 'dark-green';
  $config->{graph}->{"clientConnRefused"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"clientConnRefused"}->{"y-axis"} = '"# of RefusedConnections"';
  $config->{graph}->{"clientConnRefused"}->{"legend"} = '"Refused Connections"';
  $config->{graph}->{"clientConnRefused"}->{"units"} = '"Connections"';
  $config->{graph}->{"clientConnRefused"}->{"y-min"} = '0';

  #CHIRP::listInterfaces::listInterfaces ($config, $hostname, $snmp_community, 0, 1, 0, 0, "bits", []);

  $config->{target}->{"$hostname.CPU"}->{"long-desc"} = '"[CPU] Device CPU Utilization"';
  $config->{target}->{"$hostname.CPU"}->{"short-desc"} = '"CPU Utilization"';
  $config->{target}->{"$hostname.CPU"}->{"target-type"} = '"CPU"';

  $config->{target}->{"$hostname.Connections"}->{"long-desc"} = '"[Connections] Connections"';
  $config->{target}->{"$hostname.Connections"}->{"short-desc"} = '"Connections"';
  $config->{target}->{"$hostname.Connections"}->{"target-type"} = '"Connections"';

  $config->{target}->{"$hostname.EstConnections"}->{"long-desc"} = '"[EstConnections] Established Connections"';
  $config->{target}->{"$hostname.EstConnections"}->{"short-desc"} = '"Established Connections"';
  $config->{target}->{"$hostname.EstConnections"}->{"target-type"} = '"EstConnections"';

  $config->{target}->{"$hostname.UDPConnections"}->{"long-desc"} = '"[UDPConnections] Established Connections"';
  $config->{target}->{"$hostname.UDPConnections"}->{"short-desc"} = '"UDP Connections"';
  $config->{target}->{"$hostname.UDPConnections"}->{"target-type"} = '"UDPConnections"';

  $config->{target}->{"$hostname.clientConnRefused"}->{"long-desc"} = '"[Connections] Refused from Client"';
  $config->{target}->{"$hostname.clientConnRefused"}->{"short-desc"} = '"Refused Client Connections"';
  $config->{target}->{"$hostname.clientConnRefused"}->{"target-type"} = '"clientConnRefused"';

  return $config;
 
}

1; 
