# -*- perl -*-

use strict;

package CHIRP::Riverstone::SSR;

use lib "/usr/local/cricket/lib";
use lib::listInterfaces;
use snmpUtils;


sub rehash (@) {
  my @a = @_;
  my $b = {};
  my $row;
    
  foreach $row (@a) {
    my ($index, $content) = split (/:/, $row, 2);
    my ($i, $j) = split (/\./, $index, 2);
    $b->{$j}->{$i} = $content;
  }
  
  return $b;    
}

my $sysNameOID = ".1.3.6.1.2.1.1.5.0";

sub GenConfig {
  my($hostname, $snmp_community, $snmp_port, $options) = @_;

  $snmp_community = 'public' unless defined($snmp_community);
  $snmp_port = '161' unless defined($snmp_port);

  my($snmp) = "$snmp_community\@$hostname:$snmp_port";
  my($config) = {};
  
  my $row;

  my ($sysname) = snmpUtils::snmpget ($snmp, $sysNameOID);
  
  $config->{target}->{"--default--"}->{"rrd-datafile"} =	'%dataDir%/%auto-target-name%.rrd';
  # $config->{target}->{"--default--"}->{"router"} =	 	'%auto-target-name%';
  $config->{target}->{"--default--"}->{"router"} =	 	"\"$hostname\"";
  $config->{target}->{"--default--"}->{"snmp-host"} =	 	'%router%';
  $config->{target}->{"--default--"}->{"snmp-community"} = "$snmp_community";
  $config->{target}->{"--default--"}->{"snmp-port"} = "$snmp_port";
  $config->{target}->{"--default--"}->{"target-type"} =	 	'undef';
  $config->{target}->{"--default--"}->{"snmp-version	"} =	'1';
  $config->{target}->{"--default--"}->{"inst"} =	'1';
  $config->{target}->{"--default--"}->{"directory-desc"} =	"\"[Riverstone SSR] Statistics for $hostname ($sysname)\"";

  $config->{oid}->{"capCPUCurrentUtilization"} 	=	'1.3.6.1.4.1.52.2501.1.270.2.1.1.2.1';
  $config->{oid}->{"capCPUL3Learned"} 	=	'1.3.6.1.4.1.52.2501.1.270.2.1.1.3.1';
  $config->{oid}->{"capCPUL3Aged"} 	=	'1.3.6.1.4.1.52.2501.1.270.2.1.1.4.1';
  $config->{oid}->{"capCPUL2Learned"} 	=	'1.3.6.1.4.1.52.2501.1.270.2.1.1.5.1';
  $config->{oid}->{"capCPUL2Aged"} 	=	'1.3.6.1.4.1.52.2501.1.270.2.1.1.6.1';
  $config->{oid}->{"capCPUNIAReceived"}	=	'1.3.6.1.4.1.52.2501.1.270.2.1.1.7.1';
  $config->{oid}->{"capCPUNIATransmitted"}=	'1.3.6.1.4.1.52.2501.1.270.2.1.1.8.1';

  $config->{datasource}->{"capCPUCurrentUtilization"}->{"ds-source"} = 'snmp://%snmp%/capCPUCurrentUtilization';
  $config->{datasource}->{"capCPUCurrentUtilization"}->{"rrd-ds-type"} = 'GAUGE';

  foreach ("capCPUL3Learned", "capCPUL3Aged", 
           "capCPUL2Learned", "capCPUL2Aged", 
           "capCPUNIAReceived", "capCPUNIATransmitted") {
    
    $config->{datasource}->{$_}->{"ds-source"} = 'snmp://%snmp%/' . $_;
    $config->{datasource}->{$_}->{"rrd-ds-type"} = 'COUNTER';
  }

  $config->{targettype}->{"SSR-CPU"}->{"ds"} = '"capCPUCurrentUtilization"';
  $config->{targettype}->{"SSR-CPU"}->{"view"} = '"CPU Utilization: capCPUCurrentUtilization"';

  $config->{targettype}->{"SSR-L3Flows"}->{"ds"} = '"capCPUL3Learned, capCPUL3Aged"';
  $config->{targettype}->{"SSR-L3Flows"}->{"view"} = '"L3Flows: capCPUL3Learned capCPUL3Aged"';

  $config->{targettype}->{"SSR-L2Flows"}->{"ds"} = '"capCPUL2Learned, capCPUL2Aged"';
  $config->{targettype}->{"SSR-L2Flows"}->{"view"} = '"L2Flows: capCPUL2Learned capCPUL2Aged"';

  $config->{targettype}->{"SSR-NIA"}->{"ds"} = '"capCPUNIAReceived, capCPUNIATransmitted"';
  $config->{targettype}->{"SSR-NIA"}->{"view"} = '"CPU Interrupts: capCPUNIAReceived capCPUNIATransmitted"';

  $config->{graph}->{"capCPUNIAReceived"}->{"color"} = 'dark-green';
  $config->{graph}->{"capCPUNIAReceived"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"capCPUNIAReceived"}->{"y-axis"} = '"CPU RX Interrupts"';
  $config->{graph}->{"capCPUNIAReceived"}->{"legend"} = '"Number of CPU NIA Received"';
  $config->{graph}->{"capCPUNIAReceived"}->{"units"} = '"Intr/s"';
  $config->{graph}->{"capCPUNIAReceived"}->{"si-units"} = 'false';
  $config->{graph}->{"capCPUNIAReceived"}->{"y-min"} = '0';

  $config->{graph}->{"capCPUNIATransmitted"}->{"color"} = 'dark-blue';
  $config->{graph}->{"capCPUNIATransmitted"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"capCPUNIATransmitted"}->{"y-axis"} = '"CPU TX Interrupts"';
  $config->{graph}->{"capCPUNIATransmitted"}->{"legend"} = '"Number of CPU NIA Transmitted"';
  $config->{graph}->{"capCPUNIATransmitted"}->{"units"} = '"Intr/s"';
  $config->{graph}->{"capCPUNIATransmitted"}->{"si-units"} = 'false';
  $config->{graph}->{"capCPUNIATransmitted"}->{"y-min"} = '0';

  $config->{graph}->{"capCPUL3Aged"}->{"color"} = 'dark-blue';
  $config->{graph}->{"capCPUL3Aged"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"capCPUL3Aged"}->{"y-axis"} = '"L3 Aged Flows"';
  $config->{graph}->{"capCPUL3Aged"}->{"legend"} = '"Number of Aged L3 Flows"';
  $config->{graph}->{"capCPUL3Aged"}->{"units"} = '"Flows/s"';
  $config->{graph}->{"capCPUL3Aged"}->{"si-units"} = 'false';
  $config->{graph}->{"capCPUL3Aged"}->{"y-min"} = '0';

  $config->{graph}->{"capCPUL3Learned"}->{"color"} = 'dark-green';
  $config->{graph}->{"capCPUL3Learned"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"capCPUL3Learned"}->{"y-axis"} = '"L3 Learned Flows"';
  $config->{graph}->{"capCPUL3Learned"}->{"legend"} = '"Number of Learned L3 Flows"';
  $config->{graph}->{"capCPUL3Learned"}->{"units"} = '"Flows/s"';
  $config->{graph}->{"capCPUL3Learned"}->{"si-units"} = 'false';
  $config->{graph}->{"capCPUL3Learned"}->{"y-min"} = '0';

  $config->{graph}->{"capCPUL2Aged"}->{"color"} = 'dark-blue';
  $config->{graph}->{"capCPUL2Aged"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"capCPUL2Aged"}->{"y-axis"} = '"L2 Aged Flows"';
  $config->{graph}->{"capCPUL2Aged"}->{"legend"} = '"Number of Aged L3 Flows"';
  $config->{graph}->{"capCPUL2Aged"}->{"units"} = '"Flows/s"';
  $config->{graph}->{"capCPUL2Aged"}->{"si-units"} = 'false';
  $config->{graph}->{"capCPUL2Aged"}->{"y-min"} = '0';

  $config->{graph}->{"capCPUL2Learned"}->{"color"} = 'dark-green';
  $config->{graph}->{"capCPUL2Learned"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"capCPUL2Learned"}->{"y-axis"} = '"L2 Learned Flows"';
  $config->{graph}->{"capCPUL2Learned"}->{"legend"} = '"Number of Learned L2 Flows"';
  $config->{graph}->{"capCPUL2Learned"}->{"units"} = '"Flows/s"';
  $config->{graph}->{"capCPUL2Learned"}->{"si-units"} = 'false';
  $config->{graph}->{"capCPUL2Learned"}->{"y-min"} = '0';

  $config->{graph}->{"capCPUCurrentUtilization"}->{"color"} = 'dark-green';
  $config->{graph}->{"capCPUCurrentUtilization"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"capCPUCurrentUtilization"}->{"y-axis"} = '"Percent"';
  $config->{graph}->{"capCPUCurrentUtilization"}->{"legend"} = '"Relative Memory Utilization"';
  $config->{graph}->{"capCPUCurrentUtilization"}->{"units"} = '"%"';
  $config->{graph}->{"capCPUCurrentUtilization"}->{"si-units"} = 'false';
  $config->{graph}->{"capCPUCurrentUtilization"}->{"y-min"} = '0';

  CHIRP::listInterfaces::listInterfaces ($config, $hostname, $snmp_community, 0, 1, 0, 0, "bits", []);

  $config->{target}->{"$hostname.CPU"}->{"long-desc"} = '"[CPU] Device CPU Utilization"';
  $config->{target}->{"$hostname.CPU"}->{"short-desc"} = '"CPU Utilization"';
  $config->{target}->{"$hostname.CPU"}->{"target-type"} = '"ssr-cpu"';

  $config->{target}->{"$hostname.L3"}->{"long-desc"} = '"Layer 3 Flows Statistics"';
  $config->{target}->{"$hostname.L3"}->{"short-desc"} = '"L3 Flows"';
  $config->{target}->{"$hostname.L3"}->{"target-type"} = '"ssr-l3flows"';

  $config->{target}->{"$hostname.L2"}->{"long-desc"} = '"Layer 2 Flows Statistics"';
  $config->{target}->{"$hostname.L2"}->{"short-desc"} = '"L2 Flows"';
  $config->{target}->{"$hostname.L2"}->{"target-type"} = '"ssr-l2flows"';

  $config->{target}->{"$hostname.NIA"}->{"long-desc"} = '"SSR NIA Statistics"';
  $config->{target}->{"$hostname.NIA"}->{"short-desc"} = '"NIA"';
  $config->{target}->{"$hostname.NIA"}->{"target-type"} = '"ssr-nia"';

  return $config;
 
}

1; 
