# -*- perl -*-

use strict;

package CHIRP::SonicWall::SSL;

use lib "/usr/local/cricket/lib";
use lib::listInterfaces;
use snmpUtils;

my $sysNameOID = ".1.3.6.1.2.1.1.5.0";

sub GenConfig {
  my($hostname, $snmp_community, $snmp_port, $options) = @_;

  $snmp_community = 'public' unless defined($snmp_community);
  $snmp_port = '161' unless defined($snmp_port);

  my($snmp) = "$snmp_community\@$hostname:$snmp_port";
  my($config) = {};
  
  my ($sysname) = snmpUtils::snmpget ($snmp, $sysNameOID);
  
  $config->{target}->{"--default--"}->{"rrd-datafile"} =	'%dataDir%/%auto-target-name%.rrd';
  # $config->{target}->{"--default--"}->{"router"} =	 	'%auto-target-name%';
  $config->{target}->{"--default--"}->{"router"} =	 	"\"$hostname\"";
  $config->{target}->{"--default--"}->{"snmp-host"} =	 	'%router%';
  $config->{target}->{"--default--"}->{"snmp-community"} = "$snmp_community";
  $config->{target}->{"--default--"}->{"snmp-port"} = "$snmp_port";
  $config->{target}->{"--default--"}->{"target-type"} =	 	'sonicwall-ssl';
  $config->{target}->{"--default--"}->{"snmp-version	"} =	'1';
  $config->{target}->{"--default--"}->{"directory-desc"} =	"\"[SonicWall SSL] Statistics for $hostname ($sysname)\"";

  $config->{oid}->{"cpuUtilStatus"} 		=	'1.3.6.1.4.1.8714.2.1.6.2.1.0';
  $config->{oid}->{"totalRSAOperations"} 	=	'1.3.6.1.4.1.8714.2.1.7.2.20.0';
  $config->{oid}->{"connRejNoRes"} 		=	'1.3.6.1.4.1.8714.2.1.7.2.21.0';
  $config->{oid}->{"activeConnBlock"} 		=	'1.3.6.1.4.1.8714.2.1.7.2.22.0';
  $config->{oid}->{"activeClientConns"}		=	'1.3.6.1.4.1.8714.2.1.7.2.23.0';
  $config->{oid}->{"connAccepted"} 		=	'1.3.6.1.4.1.8714.2.1.7.2.24.0';
  $config->{oid}->{"activeServerConns"}		=	'1.3.6.1.4.1.8714.2.1.7.2.25.0';
  $config->{oid}->{"sslNegRefused"} 		=	'1.3.6.1.4.1.8714.2.1.7.2.31.0';
  $config->{oid}->{"sslNegAccept"} 		=	'1.3.6.1.4.1.8714.2.1.7.2.32.0';
  $config->{oid}->{"activeOpenSockets"} 	=	'1.3.6.1.4.1.8714.2.1.7.2.33.0';
  $config->{oid}->{"connErrs"} 			=	'1.3.6.1.4.1.8714.2.1.7.3.26.0';
  $config->{oid}->{"connBlockErrs"} 		=	'1.3.6.1.4.1.8714.2.1.7.3.27.0';
  $config->{oid}->{"sslNegErrs"} 		=	'1.3.6.1.4.1.8714.2.1.7.3.34.0';
  $config->{oid}->{"sslErrors"} 		=	'1.3.6.1.4.1.8714.2.1.7.3.35.0';
  $config->{oid}->{"openSSLAcceptErrs"} 	=	'1.3.6.1.4.1.8714.2.1.7.3.36.0';
  $config->{oid}->{"openSSLConnErrs"} 		=	'1.3.6.1.4.1.8714.2.1.7.3.37.0';
  $config->{oid}->{"totSocketErrs"} 		=	'1.3.6.1.4.1.8714.2.1.7.3.38.0';
  $config->{oid}->{"openSSLWriteErr"}	 	=	'1.3.6.1.4.1.8714.2.1.7.3.41.0';
  $config->{oid}->{"sslPipeWriteErr"}	 	=	'1.3.6.1.4.1.8714.2.1.7.3.42.0';
  $config->{oid}->{"toServerWriteErr"} 		=	'1.3.6.1.4.1.8714.2.1.7.3.43.0';
  $config->{oid}->{"toServerPipeWriteErr"} 	=	'1.3.6.1.4.1.8714.2.1.7.3.44.0';
  $config->{oid}->{"openSSLReadErr"} 		=	'1.3.6.1.4.1.8714.2.1.7.3.45.0';
  $config->{oid}->{"sslPipeReadErr"} 		=	'1.3.6.1.4.1.8714.2.1.7.3.46.0';
  $config->{oid}->{"toServerReadErr"} 		=	'1.3.6.1.4.1.8714.2.1.7.3.47.0';
  $config->{oid}->{"toServerPipeReadErr"} 	=	'1.3.6.1.4.1.8714.2.1.7.3.48.0';

  foreach ("cpuUtilStatus", "activeConnBlock", "activeClientConns", "activeServerConns", "activeOpenSockets") {
    $config->{datasource}->{$_}->{"ds-source"} = 'snmp://%snmp%/' . $_;
    $config->{datasource}->{$_}->{"rrd-ds-type"} = 'GAUGE';
  }
  foreach ("totalRSAOperations", "connRejNoRes", "connAccepted", "sslNegRefused", "sslNegAccept", 
           "connErrs", "connBlockErrs", "sslNegErrs", "sslErrors", "openSSLAcceptErrs", "openSSLConnErrs", 
           "totSocketErrs", "openSSLWriteErr", "sslPipeWriteErr", "toServerWriteErr", "toServerPipeWriteErr", 
           "openSSLReadErr", "sslPipeReadErr", "toServerReadErr", "toServerPipeReadErr") {
    $config->{datasource}->{$_}->{"ds-source"} = 'snmp://%snmp%/' . $_;
    $config->{datasource}->{$_}->{"rrd-ds-type"} = 'COUNTER';
  }

  $config->{datasource}->{"cpuUtilStatus"}->{"desc"} = 'CPU Utilization for the device';
  $config->{datasource}->{"totalRSAOperations"}->{"desc"} = 'The total number of RSA operations performed';
  $config->{datasource}->{"connRejNoRes"}->{"desc"} = 'SSL connections rejected due to low resources';
  $config->{datasource}->{"activeConnBlock"}->{"desc"} = 'Active SSL connection blocks, equivalent to the number of active client SSL connections';
  $config->{datasource}->{"activeClientConns"}->{"desc"} = 'Active client SSL connections';
  $config->{datasource}->{"connAccepted"}->{"desc"} = 'Total SSL connections accepted';
  $config->{datasource}->{"activeServerConns"}->{"desc"} = 'Active backend webserver connections';
  $config->{datasource}->{"sslNegRefused"}->{"desc"} = 'SSL Connection negotiations refused';
  $config->{datasource}->{"sslNegAccept"}->{"desc"} = 'Number of successful SSL socket negotiations';
  $config->{datasource}->{"activeOpenSockets"}->{"desc"} = 'Active SSL related open socket count. Includes client, server, and listening sockets';
  $config->{datasource}->{"connErrs"}->{"desc"} = 'Connection failures to the remote server';
  $config->{datasource}->{"connBlockErrs"}->{"desc"} = 'SSL Connection Block errors';
  $config->{datasource}->{"sslNegErrs"}->{"desc"} = 'SSL Connection Negotiation errors';
  $config->{datasource}->{"sslErrors"}->{"desc"} = 'Total SSL errors';
  $config->{datasource}->{"openSSLAcceptErrs"}->{"desc"} = 'Total SSL accept errors';
  $config->{datasource}->{"openSSLConnErrs"}->{"desc"} = 'Total SSL connection errors';
  $config->{datasource}->{"totSocketErrs"}->{"desc"} = 'Total SSL socket errors';
  $config->{datasource}->{"openSSLWriteErr"}->{"desc"} = 'SSL client write errors';
  $config->{datasource}->{"sslPipeWriteErr"}->{"desc"} = 'SSL client pipe write errors';
  $config->{datasource}->{"toServerWriteErr"}->{"desc"} = 'Backend server write errors';
  $config->{datasource}->{"toServerPipeWriteErr"}->{"desc"} = 'Backend server write pipe errors';
  $config->{datasource}->{"openSSLReadErr"}->{"desc"} = 'SSL client read errors';
  $config->{datasource}->{"sslPipeReadErr"}->{"desc"} = 'SSL client read pipe errors';
  $config->{datasource}->{"toServerReadErr"}->{"desc"} = 'Backend server read errors';
  $config->{datasource}->{"toServerPipeReadErr"}->{"desc"} = 'Backend server pipe read errors';

#  $config->{targettype}->{"sonicwall-ssl"}->{"ds"} = '"totalRSAOperations,connRejNoRes,activeConnBlock,activeClientConns,connAccepted,activeServerConns,sslNegRefused,sslNegAccept,activeOpenSockets,connErrs,connBlockErrs,sslNegErrs,sslErrors,openSSLAcceptErrs,openSSLConnErrs,totSocketErrs,openSSLWriteErr,sslPipeWriteErr,toServerWriteErr,toServerPipeWriteErr,openSSLReadErr,sslPipeReadErr,toServerReadErr,toServerPipeReadErr"';
  $config->{targettype}->{"sonicwall-ssl"}->{"ds"} = '"cpuUtilStatus,totalRSAOperations,connRejNoRes,activeConnBlock,activeClientConns,connAccepted,activeServerConns,sslNegRefused,sslNegAccept,activeOpenSockets,connErrs,connBlockErrs,sslNegErrs,sslErrors"';

  $config->{targettype}->{"sonicwall-ssl"}->{"view"} = '"CPU:cpuUtilStatus, RSA Ops:totalRSAOperations,Total Connections: connAccepted sslNegAccept,Active Connections: activeConnBlock activeClientConns activeServerConns activeOpenSockets,Rejected Connections: connRejNoRes sslNegRefused,Connection Errors: sslErrors sslNegErrs connBlockErrs connErrs"';

  $config->{graph}->{"cpuUtilStatus"}->{"color"} = 'dark-green';
  $config->{graph}->{"cpuUtilStatus"}->{"draw-as"} = 'LINE1';
  $config->{graph}->{"cpuUtilStatus"}->{"y-axis"} = '"% CPU"';
  $config->{graph}->{"cpuUtilStatus"}->{"legend"} = '"CPU Utilization"';
  $config->{graph}->{"cpuUtilStatus"}->{"units"} = '"%"';
  $config->{graph}->{"cpuUtilStatus"}->{"si-units"} = 'false';
  $config->{graph}->{"cpuUtilStatus"}->{"y-min"} = '0';

  $config->{graph}->{"totalRSAOperations"}->{"color"} = 'dark-green';
  $config->{graph}->{"totalRSAOperations"}->{"draw-as"} = 'LINE1';
  $config->{graph}->{"totalRSAOperations"}->{"y-axis"} = '"RSA Ops"';
  $config->{graph}->{"totalRSAOperations"}->{"legend"} = '"Number of RSA Operations';
  $config->{graph}->{"totalRSAOperations"}->{"units"} = '#';
  $config->{graph}->{"totalRSAOperations"}->{"si-units"} = 'false';
  $config->{graph}->{"totalRSAOperations"}->{"y-min"} = '0';

  $config->{graph}->{"connAccepted"}->{"color"} = 'dark-green';
  $config->{graph}->{"connAccepted"}->{"draw-as"} = 'LINE1';
  $config->{graph}->{"connAccepted"}->{"y-axis"} = '"# Connections"';
  $config->{graph}->{"connAccepted"}->{"legend"} = '"Total SSL connections accepted"';
  $config->{graph}->{"connAccepted"}->{"units"} = '"#"';
  $config->{graph}->{"connAccepted"}->{"si-units"} = 'false';
  $config->{graph}->{"connAccepted"}->{"y-min"} = '0';

  $config->{graph}->{"sslNegAccept"}->{"color"} = 'dark-blue';
  $config->{graph}->{"sslNegAccept"}->{"draw-as"} = 'LINE1';
  $config->{graph}->{"sslNegAccept"}->{"y-axis"} = '"# SSL socket neg"';
  $config->{graph}->{"sslNegAccept"}->{"legend"} = '"# successful SSL socket negotiations"';
  $config->{graph}->{"sslNegAccept"}->{"units"} = '"#"';
  $config->{graph}->{"sslNegAccept"}->{"si-units"} = 'false';
  $config->{graph}->{"sslNegAccept"}->{"y-min"} = '0';

  $config->{graph}->{"activeConnBlock"}->{"color"} = 'dark-green';
  $config->{graph}->{"activeConnBlock"}->{"draw-as"} = 'LINE1';
  $config->{graph}->{"activeConnBlock"}->{"y-axis"} = '"# Connection blocks"';
  $config->{graph}->{"activeConnBlock"}->{"legend"} = '"Active SSL connection blocks"';
  $config->{graph}->{"activeConnBlock"}->{"units"} = '"#"';
  $config->{graph}->{"activeConnBlock"}->{"si-units"} = 'false';
  $config->{graph}->{"activeConnBlock"}->{"y-min"} = '0';

  $config->{graph}->{"activeClientConns"}->{"color"} = 'yellow';
  $config->{graph}->{"activeClientConns"}->{"draw-as"} = 'LINE1';
  $config->{graph}->{"activeClientConns"}->{"y-axis"} = '"# Connections"';
  $config->{graph}->{"activeClientConns"}->{"legend"} = '"Active client SSL connections"';
  $config->{graph}->{"activeClientConns"}->{"units"} = '"#"';
  $config->{graph}->{"activeClientConns"}->{"si-units"} = 'false';
  $config->{graph}->{"activeClientConns"}->{"y-min"} = '0';

  $config->{graph}->{"activeServerConns"}->{"color"} = 'dark-blue';
  $config->{graph}->{"activeServerConns"}->{"draw-as"} = 'LINE1';
  $config->{graph}->{"activeServerConns"}->{"y-axis"} = '"# Connections"';
  $config->{graph}->{"activeServerConns"}->{"legend"} = '"Active backend webserver connections"';
  $config->{graph}->{"activeServerConns"}->{"units"} = '"#"';
  $config->{graph}->{"activeServerConns"}->{"si-units"} = 'false';
  $config->{graph}->{"activeServerConns"}->{"y-min"} = '0';

  $config->{graph}->{"activeOpenSockets"}->{"color"} = 'dark-red';
  $config->{graph}->{"activeOpenSockets"}->{"draw-as"} = 'LINE1';
  $config->{graph}->{"activeOpenSockets"}->{"y-axis"} = '"# Open sockets"';
  $config->{graph}->{"activeOpenSockets"}->{"legend"} = '"Active SSL related open socket count"';
  $config->{graph}->{"activeOpenSockets"}->{"units"} = '"#"';
  $config->{graph}->{"activeOpenSockets"}->{"si-units"} = 'false';
  $config->{graph}->{"activeOpenSockets"}->{"y-min"} = '0';

  $config->{graph}->{"connRejNoRes"}->{"color"} = 'red';
  $config->{graph}->{"connRejNoRes"}->{"draw-as"} = 'LINE1';
  $config->{graph}->{"connRejNoRes"}->{"y-axis"} = '"# Rejected conn"';
  $config->{graph}->{"connRejNoRes"}->{"legend"} = '"SSL connections rejected due to low resources"';
  $config->{graph}->{"connRejNoRes"}->{"units"} = '"#"';
  $config->{graph}->{"connRejNoRes"}->{"si-units"} = 'false';
  $config->{graph}->{"connRejNoRes"}->{"y-min"} = '0';

  $config->{graph}->{"sslNegRefused"}->{"color"} = 'dark-green';
  $config->{graph}->{"sslNegRefused"}->{"draw-as"} = 'LINE1';
  $config->{graph}->{"sslNegRefused"}->{"y-axis"} = '"# SSL Connection negotiations refused"';
  $config->{graph}->{"sslNegRefused"}->{"legend"} = '"SSL Connection negotiations refused"';
  $config->{graph}->{"sslNegRefused"}->{"units"} = '"#"';
  $config->{graph}->{"sslNegRefused"}->{"si-units"} = 'false';
  $config->{graph}->{"sslNegRefused"}->{"y-min"} = '0';

  $config->{graph}->{"sslErrors"}->{"color"} = 'dark-green';
  $config->{graph}->{"sslErrors"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"sslErrors"}->{"y-axis"} = '"SSL errors"';
  $config->{graph}->{"sslErrors"}->{"legend"} = '"Total SSL errors"';
  $config->{graph}->{"sslErrors"}->{"units"} = '"#"';
  $config->{graph}->{"sslErrors"}->{"si-units"} = 'false';
  $config->{graph}->{"sslErrors"}->{"y-min"} = '0';

  $config->{graph}->{"sslNegErrs"}->{"color"} = 'dark-bule';
  $config->{graph}->{"sslNegErrs"}->{"draw-as"} = 'LINE1';
  $config->{graph}->{"sslNegErrs"}->{"y-axis"} = '"# SSL Connection Negotiation errors"';
  $config->{graph}->{"sslNegErrs"}->{"legend"} = '"SSL Connection Negotiation errors"';
  $config->{graph}->{"sslNegErrs"}->{"units"} = '"#"';
  $config->{graph}->{"sslNegErrs"}->{"si-units"} = 'false';
  $config->{graph}->{"sslNegErrs"}->{"y-min"} = '0';

  $config->{graph}->{"connBlockErrs"}->{"color"} = 'dark-red';
  $config->{graph}->{"connBlockErrs"}->{"draw-as"} = 'LINE1';
  $config->{graph}->{"connBlockErrs"}->{"y-axis"} = '"# SSL Connection Block errors"';
  $config->{graph}->{"connBlockErrs"}->{"legend"} = '"SSL Connection Block errors"';
  $config->{graph}->{"connBlockErrs"}->{"units"} = '"#"';
  $config->{graph}->{"connBlockErrs"}->{"si-units"} = 'false';
  $config->{graph}->{"connBlockErrs"}->{"y-min"} = '0';

  $config->{graph}->{"connErrs"}->{"color"} = 'dark-yellow';
  $config->{graph}->{"connErrs"}->{"draw-as"} = 'LINE1';
  $config->{graph}->{"connErrs"}->{"y-axis"} = '"# Connection failures"';
  $config->{graph}->{"connErrs"}->{"legend"} = '"Connection failures to the remote server"';
  $config->{graph}->{"connErrs"}->{"units"} = '"# "';
  $config->{graph}->{"connErrs"}->{"si-units"} = 'false';
  $config->{graph}->{"connErrs"}->{"y-min"} = '0';

  $config->{target}->{"$hostname.SSL"}->{"long-desc"} = '"[SSL] Device SSL Statistics"';
  $config->{target}->{"$hostname.SSL"}->{"short-desc"} = '"SSL Statistics"';
  $config->{target}->{"$hostname.SSL"}->{"target-type"} = '"sonicwall-ssl"';

  return $config;
 
}

1; 
