# -*- perl -*-

package CHIRP::listInterfaces;
use strict;

use snmpUtils;

sub output ($$$$);
sub fmispeed ($$);
sub listInterfaces ($$$$$$$$$);

sub listInterfaces ($$$$$$$$$) {

my ($config, $router, $community, $use_64, $showspeed, $useifnamekey, $useifdescr, $units, $suppress) = @_;
my $snmp  = "$community\@$router";
my ($inoct, $outoct, $iface, $subiface);
my $frameUsed = 0;

if ($units eq '') {
  $units = 'bits';
}

if ($use_64) {
  $inoct = "ifHCInOctets";
  $outoct = "ifHCOutOctets";
  $iface = "standard-interface-64";
  $subiface = "sub-interface-64";
  $config->{targettype}->{"standard-interface-64"}->{"snmp-version"} = "2c";
  $config->{targettype}->{"sub-interface-64"}->{"snmp-version"} = "2c";
  $config->{oid}->{"ifHCInOctets"} = '1.3.6.1.2.1.31.1.1.1.6';
  $config->{oid}->{"ifHCOutOctets"} = '1.3.6.1.2.1.31.1.1.1.10';
  $config->{datasource}->{"ifHCInOctets"}->{"ds-source"} = 'snmp://%snmp%/ifHCInOctets.%inst%';
  $config->{datasource}->{"ifHCInOctets"}->{"rrd-ds-type"} = 'counter';
  $config->{datasource}->{"ifHCOutOctets"}->{"ds-source"} = 'snmp://%snmp%/ifHCOutOctets.%inst%';
  $config->{datasource}->{"ifHCOutOctets"}->{"rrd-ds-type"} = 'counter';
  $config->{targettype}->{"standard-interface-64"}->{"ds"} = '"ifHCInOctets, ifHCOutOctets, ifInErrors, ifOutErrors, ifInUcastPackets, ifOutUcastPackets"';
  $config->{targettype}->{"standard-interface-64"}->{"view"} = '"Octets: ifHCInOctets ifHCOutOctets, UcastPackets: ifInUcastPackets ifOutUcastPackets, Errors: ifInErrors ifOutErrors"';

  $config->{targettype}->{"sub-interface-64"}->{"ds"} = '"ifHCInOctets, ifHCOutOctets"';
  $config->{targettype}->{"sub-interface-64"}->{"view"} = '"Octets: ifHCInOctets ifHCOutOctets"';
  $config->{graph}->{"ifHCInOctets"}->{"color"} = 'dark-green';
  $config->{graph}->{"ifHCInOctets"}->{"draw-as"} = 'AREA';
  $config->{graph}->{"ifHCInOctets"}->{"legend"} = '"Average bits in"';
  $config->{graph}->{"ifHCInOctets"}->{"y-axis"} = '"bits per second"';
  $config->{graph}->{"ifHCInOctets"}->{"units"} = '"bits/sec"';
  $config->{graph}->{"ifHCInOctets"}->{"scale"} = '"8,*"';
  $config->{graph}->{"ifHCInOctets"}->{"bytes"} = 'true';

  $config->{graph}->{"ifHCOutOctets"}->{"color"} = 'blue';
  $config->{graph}->{"ifHCOutOctets"}->{"draw-as"} = 'LINE1';
  $config->{graph}->{"ifHCOutOctets"}->{"legend"} = '"Average bits out"';
  $config->{graph}->{"ifHCOutOctets"}->{"y-axis"} = '"bits per second"';
  $config->{graph}->{"ifHCOutOctets"}->{"units"} = '"bits/sec"';
  $config->{graph}->{"ifHCOutOctets"}->{"scale"} = '"8,*"';
  $config->{graph}->{"ifHCOutOctets"}->{"bytes"} = 'true';
} else {
  $config->{targettype}->{"standard-interface"}->{"snmp-version"} = "1";
  $config->{targettype}->{"sub-interface"}->{"snmp-version"} = "1";
  $inoct = "ifInOctets";
  $outoct = "ifOutOctets";
  $iface = "standard-interface";
  $subiface = "sub-interface";
  $config->{targettype}->{"standard-interface"}->{"ds"} = '"ifInOctets, ifOutOctets, ifInErrors, ifOutErrors, ifInUcastPackets, ifOutUcastPackets"';
  $config->{targettype}->{"standard-interface"}->{"view"} = '"Octets: ifInOctets ifOutOctets, UcastPackets: ifInUcastPackets ifOutUcastPackets, Errors: ifInErrors ifOutErrors"';

  $config->{targettype}->{"sub-interface"}->{"ds"} = '"ifInOctets, ifOutOctets"';
  $config->{targettype}->{"sub-interface"}->{"view"} = '"Octets: ifInOctets ifOutOctets"';
}

$config->{datasource}->{"ifInErrors"}->{"ds-source"} = 'snmp://%snmp%/ifInErrors.%inst%';
$config->{datasource}->{"ifInErrors"}->{"rrd-ds-type"} = 'counter';
$config->{datasource}->{"ifOutErrors"}->{"ds-source"} = 'snmp://%snmp%/ifInErrors.%inst%';
$config->{datasource}->{"ifOutErrors"}->{"rrd-ds-type"} = 'counter';
$config->{datasource}->{"ifInUcastPackets"}->{"ds-source"} = 'snmp://%snmp%/ifInUcastPkts.%inst%';
$config->{datasource}->{"ifInUcastPackets"}->{"rrd-ds-type"} = 'counter';
$config->{datasource}->{"ifOutUcastPackets"}->{"ds-source"} = 'snmp://%snmp%/ifOutUcastPkts.%inst%';
$config->{datasource}->{"ifOutUcastPackets"}->{"rrd-ds-type"} = 'counter';

# likewise, graph params for ifInOctets and ifOutOctets come from above

$config->{graph}->{"ifInUcastPackets"}->{"color"} = 'dark-green';
$config->{graph}->{"ifInUcastPackets"}->{"draw-as"} = 'AREA';
$config->{graph}->{"ifInUcastPackets"}->{"legend"} = '"Average num Packets In"';
$config->{graph}->{"ifInUcastPackets"}->{"y-axis"} = '"packets per second"';
$config->{graph}->{"ifInUcastPackets"}->{"units"} = '"pkt/sec"';
$config->{graph}->{"ifInUcastPackets"}->{"scale"} = '"8,*"';
$config->{graph}->{"ifInUcastPackets"}->{"bytes"} = 'true';

$config->{graph}->{"ifOutUcastPackets"}->{"color"} = 'blue';
$config->{graph}->{"ifOutUcastPackets"}->{"draw-as"} = 'LINE1';
$config->{graph}->{"ifOutUcastPackets"}->{"legend"} = '"Average num Packets Out"';
$config->{graph}->{"ifOutUcastPackets"}->{"y-axis"} = '"packets per second"';
$config->{graph}->{"ifOutUcastPackets"}->{"units"} = '"pkt/sec"';
$config->{graph}->{"ifOutUcastPackets"}->{"scale"} = '"8,*"';
$config->{graph}->{"ifOutUcastPackets"}->{"bytes"} = 'true';

$config->{graph}->{"ifInErrors"}->{"color"} = 'dark-green';
$config->{graph}->{"ifInErrors"}->{"draw-as"} = 'AREA';
$config->{graph}->{"ifInErrors"}->{"legend"} = '"Average Errors In"';
$config->{graph}->{"ifInErrors"}->{"y-axis"} = '"errors per second"';
$config->{graph}->{"ifInErrors"}->{"units"} = '"err/sec"';
$config->{graph}->{"ifInErrors"}->{"scale"} = '"8,*"';
$config->{graph}->{"ifInErrors"}->{"bytes"} = 'true';

$config->{graph}->{"ifOutErrors"}->{"color"} = 'blue';
$config->{graph}->{"ifOutErrors"}->{"draw-as"} = 'LINE1';
$config->{graph}->{"ifOutErrors"}->{"legend"} = '"Average Errors Out"';
$config->{graph}->{"ifOutErrors"}->{"y-axis"} = '"errors per second"';
$config->{graph}->{"ifOutErrors"}->{"units"} = '"err/sec"';
$config->{graph}->{"ifOutErrors"}->{"scale"} = '"8,*"';
$config->{graph}->{"ifOutErrors"}->{"bytes"} = 'true';

# output ($config, "--default--", "router", $router);
# output ($config, "--default--", "snmp-community", $community);

my $ifDescrOID 		= '.1.3.6.1.2.1.2.2.1.2';
my $ifNameOID 		= '.1.3.6.1.2.1.31.1.1.1.1';
my $ifAliasOID 		= '1.3.6.1.2.1.31.1.1.1.18';
my $ifAdminStatOID  	= '1.3.6.1.2.1.2.2.1.7';
my $ifOperStatOID  	= '1.3.6.1.2.1.2.2.1.8';
my $ifConnectorOID  	= '1.3.6.1.2.1.31.1.1.1.17';
my $ifTypeOID 		= '1.3.6.1.2.1.2.2.1.3';

my $interface_key  = ($useifnamekey)?$ifNameOID:$ifDescrOID;
my $interface_name  = ($useifdescr)?$ifDescrOID:$ifAliasOID;

ROW:
foreach (snmpUtils::walk($snmp, $interface_key)) {
  my($oid, $value) = split(/:/, $_, 2);

  $oid =~ s/$interface_key//;

  my ($adminstat) = snmpUtils::get($snmp, "$ifAdminStatOID.$oid");
  my ($operstat) = snmpUtils::get($snmp, "$ifOperStatOID.$oid");
  my ($connector) = snmpUtils::get($snmp, "$ifConnectorOID.$oid");
  my ($iftype) = snmpUtils::get($snmp, "$ifConnectorOID.$oid");

  my ($desc) = snmpUtils::get($snmp, "$interface_name.$oid");
  $desc =~ s/"/ /g;

  my($ifSpeed) = snmpUtils::get($snmp, "1.3.6.1.2.1.2.2.1.5.$oid");

  # Sanitize $desc value because current Redback SNMP implementation
  # as of 11/2000 returns corrupted garbage in ifDescr sometimes.

  $desc =~ tr/"!-~/ /cs; # convert all non-printable ASCIIs to a space

  next ROW if ($adminstat != 1 || $operstat != 1);

  # suppress array is list of strings to be pattern-matched to
  # decide whether we ignore this interface.

  foreach (@{$suppress}) {
    next ROW if ($value =~ m/$_/ );
  }

  my($target) = $value;
  $target =~ s/[\/\s:]/\_/g;

  output($config, $target, "interface-name", $value);
  output($config, $target, "short-desc", $desc);

  if ( $showspeed && $ifSpeed ) {
    output($config, $target, "if-speed", $ifSpeed );
    output($config, $target, "fmt-speed", fmispeed( $ifSpeed, $units ) );
  }

  # experience shows that subinterfaces (like those
  # representing the endpoints of PVCs riding on a
  # frame relay link) need a special config.
  # output ($config, $target, "snmp-version", "2c");

  output ($config, $target, "inst", 'map(interface-name)');

  if (defined($connector) && $connector == 2) {
# 
# this part is for frame-relay subinterfaces
# but could not make it work right
#    if ($iftype == 32) {
#      $frameUsed = 1;
#      output ($config, $target, "target-type", "frame-interface");
#    } else  { 
      output($config, $target,"target-type", $subiface);
#    }
  } else { 
    output ($config, $target, "target-type", $iface);
  }
  
}

if ($frameUsed) {
  $config->{targettype}->{"frame-interface"}->{"ds"}=		'"frInOctets, frOutOctets, ifRecBECN, ifRecFECN"';
  $config->{targettype}->{"frame-interface"}->{"view"} = 	'"Octets: frInOctets frOutOctets, ECN: ifRecBECN, ifRecFECN"';

  $config->{oid}->{"ifRecBECN"} = '.1.3.6.1.2.1.10.32.2.1.5';
  $config->{oid}->{"ifRecFECN"} = '.1.3.6.1.2.1.10.32.2.1.4';
  $config->{oid}->{"frInOctets"} = '.1.3.6.1.2.1.10.32.2.1.9';
  $config->{oid}->{"frOutOctets"} = '.1.3.6.1.2.1.10.32.2.1.7';

  $config->{datasource}->{"ifRecBECN"}->{"ds-source"} = 'snmp://%snmp%/ifRecBECN.%frinst%';
  $config->{datasource}->{"ifRecBECN"}->{"rrd-ds-type"} = 'counter';
  $config->{datasource}->{"ifRecFECN"}->{"ds-source"} = 'snmp://%snmp%/ifRecFECN.%frinst%';
  $config->{datasource}->{"ifRecFECN"}->{"rrd-ds-type"} = 'counter';
  $config->{datasource}->{"frInOctets"}->{"ds-source"} = 'snmp://%snmp%/frInOctets.%frinst%';
  $config->{datasource}->{"frInOctets"}->{"rrd-ds-type"} = 'counter';
  $config->{datasource}->{"frOutOctets"}->{"ds-source"} = 'snmp://%snmp%/frOutOctets.%frinst%';
  $config->{datasource}->{"frOutOctets"}->{"rrd-ds-type"} = 'counter';

  $config->{graph}->{"frInOctets"}->{"color"} = 'dark-green';
  $config->{graph}->{"frInOctets"}->{"draw-as"} = 'AREA';
  $config->{graph}->{"frInOctets"}->{"legend"} = '"Average bits in"';
  $config->{graph}->{"frInOctets"}->{"y-axis"} = '"bits per second"';
  $config->{graph}->{"frInOctets"}->{"units"} = '"bits/sec"';
  $config->{graph}->{"frInOctets"}->{"scale"} = '"8,*"';
  $config->{graph}->{"frInOctets"}->{"bytes"} = 'true';

  $config->{graph}->{"frOutOctets"}->{"color"} = 'blue';
  $config->{graph}->{"frOutOctets"}->{"draw-as"} = 'LINE1';
  $config->{graph}->{"frOutOctets"}->{"legend"} = '"Average bits out"';
  $config->{graph}->{"frOutOctets"}->{"y-axis"} = '"bits per second"';
  $config->{graph}->{"frOutOctets"}->{"units"} = '"bits/sec"';
  $config->{graph}->{"frOutOctets"}->{"scale"} = '"8,*"';
  $config->{graph}->{"frOutOctets"}->{"bytes"} = 'true';

  $config->{graph}->{"ifRecBECN"}->{"color"} = 'dark-green';
  $config->{graph}->{"ifRecBECN"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"ifRecBECN"}->{"legend"} = '"Average num of BECNs"';
  $config->{graph}->{"ifRecBECN"}->{"y-axis"} = '"Frame-relay BECN"';
  $config->{graph}->{"ifRecBECN"}->{"units"} = '"BECN"';
  $config->{graph}->{"ifRecBECN"}->{"si-units"} = 'false';
  $config->{graph}->{"ifRecBECN"}->{"precision"} = 'integer';

  $config->{graph}->{"ifRecFECN"}->{"color"} = 'blue';
  $config->{graph}->{"ifRecFECN"}->{"draw-as"} = 'LINE2';
  $config->{graph}->{"ifRecFECN"}->{"legend"} = '"Average num of FECNs"';
  $config->{graph}->{"ifRecFECN"}->{"y-axis"} = '"Frame-relay FECN"';
  $config->{graph}->{"ifRecFECN"}->{"units"} = '"FECNs"';
  $config->{graph}->{"ifRecFECN"}->{"si-units"} = 'false';
  $config->{graph}->{"ifRecFECN"}->{"precision"} = 'integer';
} 

} # listinterfaces

sub output ($$$$) {
  my($config, $target, $name, $value) = @_;

  return unless defined ($value);

  # quote empty (or white-space only) lines, or lines which
  # will have embedded spaces.

  my $quote = ($value =~ /^\s*$/ || $value =~ /\s/)?'"':'';

  $config->{target}->{$target}->{$name} = "$quote$value$quote";
}

sub fmispeed ($$) {
  my($number, $units) = @_;
  my @short;

  if ($units eq "bytes") {
      @short = ("Bytes/sec","kBytes/sec","MBytes/sec","GBytes/sec");
  } else {
      @short = ("bits/sec","kbits/sec","Mbits/sec","Gbits/sec");
  }

  my $digits = length( "".$number );
  my $divm = 0;
  while ($digits - $divm*3 > 3) { $divm++; }
  my $divnum = $number/10**($divm*3);

  return sprintf (($divnum == int $divnum)?"%1d %s":"%1.3f %s", $divnum, $short[$divm]);
}

1;
